1//=== RecordLayoutBuilder.cpp - Helper class for building record layouts ---==//
2//
3// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
4// See https://llvm.org/LICENSE.txt for license information.
5// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
6//
7//===----------------------------------------------------------------------===//
8
9#include "clang/AST/ASTContext.h"
10#include "clang/AST/ASTDiagnostic.h"
11#include "clang/AST/Attr.h"
12#include "clang/AST/CXXInheritance.h"
13#include "clang/AST/Decl.h"
14#include "clang/AST/DeclCXX.h"
15#include "clang/AST/DeclObjC.h"
16#include "clang/AST/Expr.h"
17#include "clang/AST/RecordLayout.h"
18#include "clang/AST/VTableBuilder.h"
19#include "clang/Basic/TargetInfo.h"
20#include "llvm/Support/Format.h"
21#include "llvm/Support/MathExtras.h"
22
23using namespace clang;
24
25namespace {
26
27/// BaseSubobjectInfo - Represents a single base subobject in a complete class.
28/// For a class hierarchy like
29///
30/// class A { };
31/// class B : A { };
32/// class C : A, B { };
33///
34/// The BaseSubobjectInfo graph for C will have three BaseSubobjectInfo
35/// instances, one for B and two for A.
36///
37/// If a base is virtual, it will only have one BaseSubobjectInfo allocated.
38struct BaseSubobjectInfo {
39 /// Class - The class for this base info.
40 const CXXRecordDecl *Class;
41
42 /// IsVirtual - Whether the BaseInfo represents a virtual base or not.
43 bool IsVirtual;
44
45 /// Bases - Information about the base subobjects.
46 SmallVector<BaseSubobjectInfo*, 4> Bases;
47
48 /// PrimaryVirtualBaseInfo - Holds the base info for the primary virtual base
49 /// of this base info (if one exists).
50 BaseSubobjectInfo *PrimaryVirtualBaseInfo;
51
52 // FIXME: Document.
53 const BaseSubobjectInfo *Derived;
54};
55
56/// Externally provided layout. Typically used when the AST source, such
57/// as DWARF, lacks all the information that was available at compile time, such
58/// as alignment attributes on fields and pragmas in effect.
59struct ExternalLayout {
60 ExternalLayout() = default;
61
62 /// Overall record size in bits.
63 uint64_t Size = 0;
64
65 /// Overall record alignment in bits.
66 uint64_t Align = 0;
67
68 /// Record field offsets in bits.
69 llvm::DenseMap<const FieldDecl *, uint64_t> FieldOffsets;
70
71 /// Direct, non-virtual base offsets.
72 llvm::DenseMap<const CXXRecordDecl *, CharUnits> BaseOffsets;
73
74 /// Virtual base offsets.
75 llvm::DenseMap<const CXXRecordDecl *, CharUnits> VirtualBaseOffsets;
76
77 /// Get the offset of the given field. The external source must provide
78 /// entries for all fields in the record.
79 uint64_t getExternalFieldOffset(const FieldDecl *FD) {
80 assert(FieldOffsets.count(FD) &&
81 "Field does not have an external offset");
82 return FieldOffsets[FD];
83 }
84
85 bool getExternalNVBaseOffset(const CXXRecordDecl *RD, CharUnits &BaseOffset) {
86 auto Known = BaseOffsets.find(Val: RD);
87 if (Known == BaseOffsets.end())
88 return false;
89 BaseOffset = Known->second;
90 return true;
91 }
92
93 bool getExternalVBaseOffset(const CXXRecordDecl *RD, CharUnits &BaseOffset) {
94 auto Known = VirtualBaseOffsets.find(Val: RD);
95 if (Known == VirtualBaseOffsets.end())
96 return false;
97 BaseOffset = Known->second;
98 return true;
99 }
100};
101
102/// EmptySubobjectMap - Keeps track of which empty subobjects exist at different
103/// offsets while laying out a C++ class.
104class EmptySubobjectMap {
105 const ASTContext &Context;
106 uint64_t CharWidth;
107
108 /// Class - The class whose empty entries we're keeping track of.
109 const CXXRecordDecl *Class;
110
111 /// EmptyClassOffsets - A map from offsets to empty record decls.
112 typedef llvm::TinyPtrVector<const CXXRecordDecl *> ClassVectorTy;
113 typedef llvm::DenseMap<CharUnits, ClassVectorTy> EmptyClassOffsetsMapTy;
114 EmptyClassOffsetsMapTy EmptyClassOffsets;
115
116 /// MaxEmptyClassOffset - The highest offset known to contain an empty
117 /// base subobject.
118 CharUnits MaxEmptyClassOffset;
119
120 /// ComputeEmptySubobjectSizes - Compute the size of the largest base or
121 /// member subobject that is empty.
122 void ComputeEmptySubobjectSizes();
123
124 void AddSubobjectAtOffset(const CXXRecordDecl *RD, CharUnits Offset);
125
126 void UpdateEmptyBaseSubobjects(const BaseSubobjectInfo *Info,
127 CharUnits Offset, bool PlacingEmptyBase);
128
129 void UpdateEmptyFieldSubobjects(const CXXRecordDecl *RD,
130 const CXXRecordDecl *Class, CharUnits Offset,
131 bool PlacingOverlappingField);
132 void UpdateEmptyFieldSubobjects(const FieldDecl *FD, CharUnits Offset,
133 bool PlacingOverlappingField);
134
135 /// AnyEmptySubobjectsBeyondOffset - Returns whether there are any empty
136 /// subobjects beyond the given offset.
137 bool AnyEmptySubobjectsBeyondOffset(CharUnits Offset) const {
138 return Offset <= MaxEmptyClassOffset;
139 }
140
141 CharUnits getFieldOffset(const ASTRecordLayout &Layout,
142 const FieldDecl *Field) const {
143 uint64_t FieldOffset = Layout.getFieldOffset(FieldNo: Field->getFieldIndex());
144 assert(FieldOffset % CharWidth == 0 &&
145 "Field offset not at char boundary!");
146
147 return Context.toCharUnitsFromBits(BitSize: FieldOffset);
148 }
149
150protected:
151 bool CanPlaceSubobjectAtOffset(const CXXRecordDecl *RD,
152 CharUnits Offset) const;
153
154 bool CanPlaceBaseSubobjectAtOffset(const BaseSubobjectInfo *Info,
155 CharUnits Offset);
156
157 bool CanPlaceFieldSubobjectAtOffset(const CXXRecordDecl *RD,
158 const CXXRecordDecl *Class,
159 CharUnits Offset) const;
160 bool CanPlaceFieldSubobjectAtOffset(const FieldDecl *FD,
161 CharUnits Offset) const;
162
163public:
164 /// This holds the size of the largest empty subobject (either a base
165 /// or a member). Will be zero if the record being built doesn't contain
166 /// any empty classes.
167 CharUnits SizeOfLargestEmptySubobject;
168
169 EmptySubobjectMap(const ASTContext &Context, const CXXRecordDecl *Class)
170 : Context(Context), CharWidth(Context.getCharWidth()), Class(Class) {
171 ComputeEmptySubobjectSizes();
172 }
173
174 /// CanPlaceBaseAtOffset - Return whether the given base class can be placed
175 /// at the given offset.
176 /// Returns false if placing the record will result in two components
177 /// (direct or indirect) of the same type having the same offset.
178 bool CanPlaceBaseAtOffset(const BaseSubobjectInfo *Info,
179 CharUnits Offset);
180
181 /// CanPlaceFieldAtOffset - Return whether a field can be placed at the given
182 /// offset.
183 bool CanPlaceFieldAtOffset(const FieldDecl *FD, CharUnits Offset);
184};
185
186void EmptySubobjectMap::ComputeEmptySubobjectSizes() {
187 // Check the bases.
188 for (const CXXBaseSpecifier &Base : Class->bases()) {
189 const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
190 assert(BaseDecl != Class && "Class cannot inherit from itself.");
191
192 CharUnits EmptySize;
193 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: BaseDecl);
194 if (BaseDecl->isEmpty()) {
195 // If the class decl is empty, get its size.
196 EmptySize = Layout.getSize();
197 } else {
198 // Otherwise, we get the largest empty subobject for the decl.
199 EmptySize = Layout.getSizeOfLargestEmptySubobject();
200 }
201
202 if (EmptySize > SizeOfLargestEmptySubobject)
203 SizeOfLargestEmptySubobject = EmptySize;
204 }
205
206 // Check the fields.
207 for (const FieldDecl *FD : Class->fields()) {
208 // We only care about records.
209 const auto *MemberDecl =
210 Context.getBaseElementType(QT: FD->getType())->getAsCXXRecordDecl();
211 if (!MemberDecl)
212 continue;
213
214 CharUnits EmptySize;
215 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: MemberDecl);
216 if (MemberDecl->isEmpty()) {
217 // If the class decl is empty, get its size.
218 EmptySize = Layout.getSize();
219 } else {
220 // Otherwise, we get the largest empty subobject for the decl.
221 EmptySize = Layout.getSizeOfLargestEmptySubobject();
222 }
223
224 if (EmptySize > SizeOfLargestEmptySubobject)
225 SizeOfLargestEmptySubobject = EmptySize;
226 }
227}
228
229bool
230EmptySubobjectMap::CanPlaceSubobjectAtOffset(const CXXRecordDecl *RD,
231 CharUnits Offset) const {
232 // We only need to check empty bases.
233 if (!RD->isEmpty())
234 return true;
235
236 EmptyClassOffsetsMapTy::const_iterator I = EmptyClassOffsets.find(Val: Offset);
237 if (I == EmptyClassOffsets.end())
238 return true;
239
240 const ClassVectorTy &Classes = I->second;
241 if (!llvm::is_contained(Range: Classes, Element: RD))
242 return true;
243
244 // There is already an empty class of the same type at this offset.
245 return false;
246}
247
248void EmptySubobjectMap::AddSubobjectAtOffset(const CXXRecordDecl *RD,
249 CharUnits Offset) {
250 // We only care about empty bases.
251 if (!RD->isEmpty())
252 return;
253
254 // If we have empty structures inside a union, we can assign both
255 // the same offset. Just avoid pushing them twice in the list.
256 ClassVectorTy &Classes = EmptyClassOffsets[Offset];
257 if (llvm::is_contained(Range&: Classes, Element: RD))
258 return;
259
260 Classes.push_back(NewVal: RD);
261
262 // Update the empty class offset.
263 if (Offset > MaxEmptyClassOffset)
264 MaxEmptyClassOffset = Offset;
265}
266
267bool
268EmptySubobjectMap::CanPlaceBaseSubobjectAtOffset(const BaseSubobjectInfo *Info,
269 CharUnits Offset) {
270 // We don't have to keep looking past the maximum offset that's known to
271 // contain an empty class.
272 if (!AnyEmptySubobjectsBeyondOffset(Offset))
273 return true;
274
275 if (!CanPlaceSubobjectAtOffset(RD: Info->Class, Offset))
276 return false;
277
278 // Traverse all non-virtual bases.
279 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: Info->Class);
280 for (const BaseSubobjectInfo *Base : Info->Bases) {
281 if (Base->IsVirtual)
282 continue;
283
284 CharUnits BaseOffset = Offset + Layout.getBaseClassOffset(Base: Base->Class);
285
286 if (!CanPlaceBaseSubobjectAtOffset(Info: Base, Offset: BaseOffset))
287 return false;
288 }
289
290 if (Info->PrimaryVirtualBaseInfo) {
291 BaseSubobjectInfo *PrimaryVirtualBaseInfo = Info->PrimaryVirtualBaseInfo;
292
293 if (Info == PrimaryVirtualBaseInfo->Derived) {
294 if (!CanPlaceBaseSubobjectAtOffset(Info: PrimaryVirtualBaseInfo, Offset))
295 return false;
296 }
297 }
298
299 // Traverse all member variables.
300 for (const FieldDecl *Field : Info->Class->fields()) {
301 if (Field->isBitField())
302 continue;
303
304 CharUnits FieldOffset = Offset + getFieldOffset(Layout, Field);
305 if (!CanPlaceFieldSubobjectAtOffset(FD: Field, Offset: FieldOffset))
306 return false;
307 }
308
309 return true;
310}
311
312void EmptySubobjectMap::UpdateEmptyBaseSubobjects(const BaseSubobjectInfo *Info,
313 CharUnits Offset,
314 bool PlacingEmptyBase) {
315 if (!PlacingEmptyBase && Offset >= SizeOfLargestEmptySubobject) {
316 // We know that the only empty subobjects that can conflict with empty
317 // subobject of non-empty bases, are empty bases that can be placed at
318 // offset zero. Because of this, we only need to keep track of empty base
319 // subobjects with offsets less than the size of the largest empty
320 // subobject for our class.
321 return;
322 }
323
324 AddSubobjectAtOffset(RD: Info->Class, Offset);
325
326 // Traverse all non-virtual bases.
327 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: Info->Class);
328 for (const BaseSubobjectInfo *Base : Info->Bases) {
329 if (Base->IsVirtual)
330 continue;
331
332 CharUnits BaseOffset = Offset + Layout.getBaseClassOffset(Base: Base->Class);
333 UpdateEmptyBaseSubobjects(Info: Base, Offset: BaseOffset, PlacingEmptyBase);
334 }
335
336 if (Info->PrimaryVirtualBaseInfo) {
337 BaseSubobjectInfo *PrimaryVirtualBaseInfo = Info->PrimaryVirtualBaseInfo;
338
339 if (Info == PrimaryVirtualBaseInfo->Derived)
340 UpdateEmptyBaseSubobjects(Info: PrimaryVirtualBaseInfo, Offset,
341 PlacingEmptyBase);
342 }
343
344 // Traverse all member variables.
345 for (const FieldDecl *Field : Info->Class->fields()) {
346 if (Field->isBitField())
347 continue;
348
349 CharUnits FieldOffset = Offset + getFieldOffset(Layout, Field);
350 UpdateEmptyFieldSubobjects(FD: Field, Offset: FieldOffset, PlacingOverlappingField: PlacingEmptyBase);
351 }
352}
353
354bool EmptySubobjectMap::CanPlaceBaseAtOffset(const BaseSubobjectInfo *Info,
355 CharUnits Offset) {
356 // If we know this class doesn't have any empty subobjects we don't need to
357 // bother checking.
358 if (SizeOfLargestEmptySubobject.isZero())
359 return true;
360
361 if (!CanPlaceBaseSubobjectAtOffset(Info, Offset))
362 return false;
363
364 // We are able to place the base at this offset. Make sure to update the
365 // empty base subobject map.
366 UpdateEmptyBaseSubobjects(Info, Offset, PlacingEmptyBase: Info->Class->isEmpty());
367 return true;
368}
369
370bool
371EmptySubobjectMap::CanPlaceFieldSubobjectAtOffset(const CXXRecordDecl *RD,
372 const CXXRecordDecl *Class,
373 CharUnits Offset) const {
374 // We don't have to keep looking past the maximum offset that's known to
375 // contain an empty class.
376 if (!AnyEmptySubobjectsBeyondOffset(Offset))
377 return true;
378
379 if (!CanPlaceSubobjectAtOffset(RD, Offset))
380 return false;
381
382 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: RD);
383
384 // Traverse all non-virtual bases.
385 for (const CXXBaseSpecifier &Base : RD->bases()) {
386 if (Base.isVirtual())
387 continue;
388
389 const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
390
391 CharUnits BaseOffset = Offset + Layout.getBaseClassOffset(Base: BaseDecl);
392 if (!CanPlaceFieldSubobjectAtOffset(RD: BaseDecl, Class, Offset: BaseOffset))
393 return false;
394 }
395
396 if (RD == Class) {
397 // This is the most derived class, traverse virtual bases as well.
398 for (const CXXBaseSpecifier &Base : RD->vbases()) {
399 const CXXRecordDecl *VBaseDecl = Base.getType()->getAsCXXRecordDecl();
400
401 CharUnits VBaseOffset = Offset + Layout.getVBaseClassOffset(VBase: VBaseDecl);
402 if (!CanPlaceFieldSubobjectAtOffset(RD: VBaseDecl, Class, Offset: VBaseOffset))
403 return false;
404 }
405 }
406
407 // Traverse all member variables.
408 for (const FieldDecl *Field : RD->fields()) {
409 if (Field->isBitField())
410 continue;
411
412 CharUnits FieldOffset = Offset + getFieldOffset(Layout, Field);
413 if (!CanPlaceFieldSubobjectAtOffset(FD: Field, Offset: FieldOffset))
414 return false;
415 }
416
417 return true;
418}
419
420bool
421EmptySubobjectMap::CanPlaceFieldSubobjectAtOffset(const FieldDecl *FD,
422 CharUnits Offset) const {
423 // We don't have to keep looking past the maximum offset that's known to
424 // contain an empty class.
425 if (!AnyEmptySubobjectsBeyondOffset(Offset))
426 return true;
427
428 QualType T = FD->getType();
429 if (const CXXRecordDecl *RD = T->getAsCXXRecordDecl())
430 return CanPlaceFieldSubobjectAtOffset(RD, Class: RD, Offset);
431
432 // If we have an array type we need to look at every element.
433 if (const ConstantArrayType *AT = Context.getAsConstantArrayType(T)) {
434 QualType ElemTy = Context.getBaseElementType(VAT: AT);
435 const auto *RD = ElemTy->getAsCXXRecordDecl();
436 if (!RD)
437 return true;
438
439 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: RD);
440
441 uint64_t NumElements = Context.getConstantArrayElementCount(CA: AT);
442 CharUnits ElementOffset = Offset;
443 for (uint64_t I = 0; I != NumElements; ++I) {
444 // We don't have to keep looking past the maximum offset that's known to
445 // contain an empty class.
446 if (!AnyEmptySubobjectsBeyondOffset(Offset: ElementOffset))
447 return true;
448
449 if (!CanPlaceFieldSubobjectAtOffset(RD, Class: RD, Offset: ElementOffset))
450 return false;
451
452 ElementOffset += Layout.getSize();
453 }
454 }
455
456 return true;
457}
458
459bool EmptySubobjectMap::CanPlaceFieldAtOffset(const FieldDecl *FD,
460 CharUnits Offset) {
461 if (!CanPlaceFieldSubobjectAtOffset(FD, Offset))
462 return false;
463
464 // We are able to place the member variable at this offset.
465 // Make sure to update the empty field subobject map.
466 UpdateEmptyFieldSubobjects(FD, Offset, PlacingOverlappingField: FD->hasAttr<NoUniqueAddressAttr>());
467 return true;
468}
469
470void EmptySubobjectMap::UpdateEmptyFieldSubobjects(
471 const CXXRecordDecl *RD, const CXXRecordDecl *Class, CharUnits Offset,
472 bool PlacingOverlappingField) {
473 // We know that the only empty subobjects that can conflict with empty
474 // field subobjects are subobjects of empty bases and potentially-overlapping
475 // fields that can be placed at offset zero. Because of this, we only need to
476 // keep track of empty field subobjects with offsets less than the size of
477 // the largest empty subobject for our class.
478 //
479 // (Proof: we will only consider placing a subobject at offset zero or at
480 // >= the current dsize. The only cases where the earlier subobject can be
481 // placed beyond the end of dsize is if it's an empty base or a
482 // potentially-overlapping field.)
483 if (!PlacingOverlappingField && Offset >= SizeOfLargestEmptySubobject)
484 return;
485
486 AddSubobjectAtOffset(RD, Offset);
487
488 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: RD);
489
490 // Traverse all non-virtual bases.
491 for (const CXXBaseSpecifier &Base : RD->bases()) {
492 if (Base.isVirtual())
493 continue;
494
495 const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
496
497 CharUnits BaseOffset = Offset + Layout.getBaseClassOffset(Base: BaseDecl);
498 UpdateEmptyFieldSubobjects(RD: BaseDecl, Class, Offset: BaseOffset,
499 PlacingOverlappingField);
500 }
501
502 if (RD == Class) {
503 // This is the most derived class, traverse virtual bases as well.
504 for (const CXXBaseSpecifier &Base : RD->vbases()) {
505 const CXXRecordDecl *VBaseDecl = Base.getType()->getAsCXXRecordDecl();
506
507 CharUnits VBaseOffset = Offset + Layout.getVBaseClassOffset(VBase: VBaseDecl);
508 UpdateEmptyFieldSubobjects(RD: VBaseDecl, Class, Offset: VBaseOffset,
509 PlacingOverlappingField);
510 }
511 }
512
513 // Traverse all member variables.
514 for (const FieldDecl *Field : RD->fields()) {
515 if (Field->isBitField())
516 continue;
517
518 CharUnits FieldOffset = Offset + getFieldOffset(Layout, Field);
519 UpdateEmptyFieldSubobjects(FD: Field, Offset: FieldOffset, PlacingOverlappingField);
520 }
521}
522
523void EmptySubobjectMap::UpdateEmptyFieldSubobjects(
524 const FieldDecl *FD, CharUnits Offset, bool PlacingOverlappingField) {
525 QualType T = FD->getType();
526 if (const CXXRecordDecl *RD = T->getAsCXXRecordDecl()) {
527 UpdateEmptyFieldSubobjects(RD, Class: RD, Offset, PlacingOverlappingField);
528 return;
529 }
530
531 // If we have an array type we need to update every element.
532 if (const ConstantArrayType *AT = Context.getAsConstantArrayType(T)) {
533 QualType ElemTy = Context.getBaseElementType(VAT: AT);
534 const auto *RD = ElemTy->getAsCXXRecordDecl();
535 if (!RD)
536 return;
537
538 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: RD);
539
540 uint64_t NumElements = Context.getConstantArrayElementCount(CA: AT);
541 CharUnits ElementOffset = Offset;
542
543 for (uint64_t I = 0; I != NumElements; ++I) {
544 // We know that the only empty subobjects that can conflict with empty
545 // field subobjects are subobjects of empty bases that can be placed at
546 // offset zero. Because of this, we only need to keep track of empty field
547 // subobjects with offsets less than the size of the largest empty
548 // subobject for our class.
549 if (!PlacingOverlappingField &&
550 ElementOffset >= SizeOfLargestEmptySubobject)
551 return;
552
553 UpdateEmptyFieldSubobjects(RD, Class: RD, Offset: ElementOffset,
554 PlacingOverlappingField);
555 ElementOffset += Layout.getSize();
556 }
557 }
558}
559
560typedef llvm::SmallPtrSet<const CXXRecordDecl*, 4> ClassSetTy;
561
562class ItaniumRecordLayoutBuilder {
563protected:
564 // FIXME: Remove this and make the appropriate fields public.
565 friend class clang::ASTContext;
566
567 const ASTContext &Context;
568
569 EmptySubobjectMap *EmptySubobjects;
570
571 /// Size - The current size of the record layout.
572 uint64_t Size;
573
574 /// Alignment - The current alignment of the record layout.
575 CharUnits Alignment;
576
577 /// PreferredAlignment - The preferred alignment of the record layout.
578 CharUnits PreferredAlignment;
579
580 /// The alignment if attribute packed is not used.
581 CharUnits UnpackedAlignment;
582
583 /// \brief The maximum of the alignments of top-level members.
584 CharUnits UnadjustedAlignment;
585
586 SmallVector<uint64_t, 16> FieldOffsets;
587
588 /// Whether the external AST source has provided a layout for this
589 /// record.
590 LLVM_PREFERRED_TYPE(bool)
591 unsigned UseExternalLayout : 1;
592
593 /// Whether we need to infer alignment, even when we have an
594 /// externally-provided layout.
595 LLVM_PREFERRED_TYPE(bool)
596 unsigned InferAlignment : 1;
597
598 /// Packed - Whether the record is packed or not.
599 LLVM_PREFERRED_TYPE(bool)
600 unsigned Packed : 1;
601
602 LLVM_PREFERRED_TYPE(bool)
603 unsigned IsUnion : 1;
604
605 LLVM_PREFERRED_TYPE(bool)
606 unsigned IsMac68kAlign : 1;
607
608 LLVM_PREFERRED_TYPE(bool)
609 unsigned IsNaturalAlign : 1;
610
611 LLVM_PREFERRED_TYPE(bool)
612 unsigned IsMsStruct : 1;
613
614 /// UnfilledBitsInLastUnit - If the last field laid out was a bitfield,
615 /// this contains the number of bits in the last unit that can be used for
616 /// an adjacent bitfield if necessary. The unit in question is usually
617 /// a byte, but larger units are used if IsMsStruct.
618 uint64_t UnfilledBitsInLastUnit;
619
620 /// LastBitfieldStorageUnitSize - If IsMsStruct, represents the size of the
621 /// storage unit of the previous field if it was a bitfield.
622 uint64_t LastBitfieldStorageUnitSize;
623
624 /// MaxFieldAlignment - The maximum allowed field alignment. This is set by
625 /// #pragma pack.
626 CharUnits MaxFieldAlignment;
627
628 /// DataSize - The data size of the record being laid out.
629 uint64_t DataSize;
630
631 CharUnits NonVirtualSize;
632 CharUnits NonVirtualAlignment;
633 CharUnits PreferredNVAlignment;
634
635 /// If we've laid out a field but not included its tail padding in Size yet,
636 /// this is the size up to the end of that field.
637 CharUnits PaddedFieldSize;
638
639 /// PrimaryBase - the primary base class (if one exists) of the class
640 /// we're laying out.
641 const CXXRecordDecl *PrimaryBase;
642
643 /// PrimaryBaseIsVirtual - Whether the primary base of the class we're laying
644 /// out is virtual.
645 bool PrimaryBaseIsVirtual;
646
647 /// HasOwnVFPtr - Whether the class provides its own vtable/vftbl
648 /// pointer, as opposed to inheriting one from a primary base class.
649 bool HasOwnVFPtr;
650
651 /// the flag of field offset changing due to packed attribute.
652 bool HasPackedField;
653
654 /// HandledFirstNonOverlappingEmptyField - An auxiliary field used for AIX.
655 /// When there are OverlappingEmptyFields existing in the aggregate, the
656 /// flag shows if the following first non-empty or empty-but-non-overlapping
657 /// field has been handled, if any.
658 bool HandledFirstNonOverlappingEmptyField;
659
660 typedef llvm::DenseMap<const CXXRecordDecl *, CharUnits> BaseOffsetsMapTy;
661
662 /// Bases - base classes and their offsets in the record.
663 BaseOffsetsMapTy Bases;
664
665 // VBases - virtual base classes and their offsets in the record.
666 ASTRecordLayout::VBaseOffsetsMapTy VBases;
667
668 /// IndirectPrimaryBases - Virtual base classes, direct or indirect, that are
669 /// primary base classes for some other direct or indirect base class.
670 CXXIndirectPrimaryBaseSet IndirectPrimaryBases;
671
672 /// FirstNearlyEmptyVBase - The first nearly empty virtual base class in
673 /// inheritance graph order. Used for determining the primary base class.
674 const CXXRecordDecl *FirstNearlyEmptyVBase;
675
676 /// VisitedVirtualBases - A set of all the visited virtual bases, used to
677 /// avoid visiting virtual bases more than once.
678 llvm::SmallPtrSet<const CXXRecordDecl *, 4> VisitedVirtualBases;
679
680 /// Valid if UseExternalLayout is true.
681 ExternalLayout External;
682
683 ItaniumRecordLayoutBuilder(const ASTContext &Context,
684 EmptySubobjectMap *EmptySubobjects)
685 : Context(Context), EmptySubobjects(EmptySubobjects), Size(0),
686 Alignment(CharUnits::One()), PreferredAlignment(CharUnits::One()),
687 UnpackedAlignment(CharUnits::One()),
688 UnadjustedAlignment(CharUnits::One()), UseExternalLayout(false),
689 InferAlignment(false), Packed(false), IsUnion(false),
690 IsMac68kAlign(false),
691 IsNaturalAlign(!Context.getTargetInfo().getTriple().isOSAIX()),
692 IsMsStruct(false), UnfilledBitsInLastUnit(0),
693 LastBitfieldStorageUnitSize(0), MaxFieldAlignment(CharUnits::Zero()),
694 DataSize(0), NonVirtualSize(CharUnits::Zero()),
695 NonVirtualAlignment(CharUnits::One()),
696 PreferredNVAlignment(CharUnits::One()),
697 PaddedFieldSize(CharUnits::Zero()), PrimaryBase(nullptr),
698 PrimaryBaseIsVirtual(false), HasOwnVFPtr(false), HasPackedField(false),
699 HandledFirstNonOverlappingEmptyField(false),
700 FirstNearlyEmptyVBase(nullptr) {}
701
702 void Layout(const RecordDecl *D);
703 void Layout(const CXXRecordDecl *D);
704 void Layout(const ObjCInterfaceDecl *D);
705
706 void LayoutFields(const RecordDecl *D);
707 void LayoutField(const FieldDecl *D, bool InsertExtraPadding);
708 void LayoutWideBitField(uint64_t FieldSize, uint64_t StorageUnitSize,
709 bool FieldPacked, const FieldDecl *D);
710 void LayoutBitField(const FieldDecl *D);
711
712 TargetCXXABI getCXXABI() const {
713 return Context.getTargetInfo().getCXXABI();
714 }
715
716 /// BaseSubobjectInfoAllocator - Allocator for BaseSubobjectInfo objects.
717 llvm::SpecificBumpPtrAllocator<BaseSubobjectInfo> BaseSubobjectInfoAllocator;
718
719 typedef llvm::DenseMap<const CXXRecordDecl *, BaseSubobjectInfo *>
720 BaseSubobjectInfoMapTy;
721
722 /// VirtualBaseInfo - Map from all the (direct or indirect) virtual bases
723 /// of the class we're laying out to their base subobject info.
724 BaseSubobjectInfoMapTy VirtualBaseInfo;
725
726 /// NonVirtualBaseInfo - Map from all the direct non-virtual bases of the
727 /// class we're laying out to their base subobject info.
728 BaseSubobjectInfoMapTy NonVirtualBaseInfo;
729
730 /// ComputeBaseSubobjectInfo - Compute the base subobject information for the
731 /// bases of the given class.
732 void ComputeBaseSubobjectInfo(const CXXRecordDecl *RD);
733
734 /// ComputeBaseSubobjectInfo - Compute the base subobject information for a
735 /// single class and all of its base classes.
736 BaseSubobjectInfo *ComputeBaseSubobjectInfo(const CXXRecordDecl *RD,
737 bool IsVirtual,
738 BaseSubobjectInfo *Derived);
739
740 /// DeterminePrimaryBase - Determine the primary base of the given class.
741 void DeterminePrimaryBase(const CXXRecordDecl *RD);
742
743 void SelectPrimaryVBase(const CXXRecordDecl *RD);
744
745 void EnsureVTablePointerAlignment(CharUnits UnpackedBaseAlign);
746
747 /// LayoutNonVirtualBases - Determines the primary base class (if any) and
748 /// lays it out. Will then proceed to lay out all non-virtual base clasess.
749 void LayoutNonVirtualBases(const CXXRecordDecl *RD);
750
751 /// LayoutNonVirtualBase - Lays out a single non-virtual base.
752 void LayoutNonVirtualBase(const BaseSubobjectInfo *Base);
753
754 void AddPrimaryVirtualBaseOffsets(const BaseSubobjectInfo *Info,
755 CharUnits Offset);
756
757 /// LayoutVirtualBases - Lays out all the virtual bases.
758 void LayoutVirtualBases(const CXXRecordDecl *RD,
759 const CXXRecordDecl *MostDerivedClass);
760
761 /// LayoutVirtualBase - Lays out a single virtual base.
762 void LayoutVirtualBase(const BaseSubobjectInfo *Base);
763
764 /// LayoutBase - Will lay out a base and return the offset where it was
765 /// placed, in chars.
766 CharUnits LayoutBase(const BaseSubobjectInfo *Base);
767
768 /// InitializeLayout - Initialize record layout for the given record decl.
769 void InitializeLayout(const Decl *D);
770
771 /// FinishLayout - Finalize record layout. Adjust record size based on the
772 /// alignment.
773 void FinishLayout(const NamedDecl *D);
774
775 void UpdateAlignment(CharUnits NewAlignment, CharUnits UnpackedNewAlignment,
776 CharUnits PreferredAlignment);
777 void UpdateAlignment(CharUnits NewAlignment, CharUnits UnpackedNewAlignment) {
778 UpdateAlignment(NewAlignment, UnpackedNewAlignment, PreferredAlignment: NewAlignment);
779 }
780 void UpdateAlignment(CharUnits NewAlignment) {
781 UpdateAlignment(NewAlignment, UnpackedNewAlignment: NewAlignment, PreferredAlignment: NewAlignment);
782 }
783
784 /// Retrieve the externally-supplied field offset for the given
785 /// field.
786 ///
787 /// \param Field The field whose offset is being queried.
788 /// \param ComputedOffset The offset that we've computed for this field.
789 uint64_t updateExternalFieldOffset(const FieldDecl *Field,
790 uint64_t ComputedOffset);
791
792 void CheckFieldPadding(uint64_t Offset, uint64_t UnpaddedOffset,
793 uint64_t UnpackedOffset, unsigned UnpackedAlign,
794 bool isPacked, const FieldDecl *D);
795
796 DiagnosticBuilder Diag(SourceLocation Loc, unsigned DiagID);
797
798 CharUnits getSize() const {
799 assert(Size % Context.getCharWidth() == 0);
800 return Context.toCharUnitsFromBits(BitSize: Size);
801 }
802 uint64_t getSizeInBits() const { return Size; }
803
804 void setSize(CharUnits NewSize) { Size = Context.toBits(CharSize: NewSize); }
805 void setSize(uint64_t NewSize) { Size = NewSize; }
806
807 CharUnits getAlignment() const { return Alignment; }
808
809 CharUnits getDataSize() const {
810 assert(DataSize % Context.getCharWidth() == 0);
811 return Context.toCharUnitsFromBits(BitSize: DataSize);
812 }
813 uint64_t getDataSizeInBits() const { return DataSize; }
814
815 void setDataSize(CharUnits NewSize) { DataSize = Context.toBits(CharSize: NewSize); }
816 void setDataSize(uint64_t NewSize) { DataSize = NewSize; }
817
818 ItaniumRecordLayoutBuilder(const ItaniumRecordLayoutBuilder &) = delete;
819 void operator=(const ItaniumRecordLayoutBuilder &) = delete;
820};
821} // end anonymous namespace
822
823void ItaniumRecordLayoutBuilder::SelectPrimaryVBase(const CXXRecordDecl *RD) {
824 for (const auto &I : RD->bases()) {
825 assert(!I.getType()->isDependentType() &&
826 "Cannot layout class with dependent bases.");
827
828 const CXXRecordDecl *Base = I.getType()->getAsCXXRecordDecl();
829
830 // Check if this is a nearly empty virtual base.
831 if (I.isVirtual() && Context.isNearlyEmpty(RD: Base)) {
832 // If it's not an indirect primary base, then we've found our primary
833 // base.
834 if (!IndirectPrimaryBases.count(Ptr: Base)) {
835 PrimaryBase = Base;
836 PrimaryBaseIsVirtual = true;
837 return;
838 }
839
840 // Is this the first nearly empty virtual base?
841 if (!FirstNearlyEmptyVBase)
842 FirstNearlyEmptyVBase = Base;
843 }
844
845 SelectPrimaryVBase(RD: Base);
846 if (PrimaryBase)
847 return;
848 }
849}
850
851/// DeterminePrimaryBase - Determine the primary base of the given class.
852void ItaniumRecordLayoutBuilder::DeterminePrimaryBase(const CXXRecordDecl *RD) {
853 // If the class isn't dynamic, it won't have a primary base.
854 if (!RD->isDynamicClass())
855 return;
856
857 // Compute all the primary virtual bases for all of our direct and
858 // indirect bases, and record all their primary virtual base classes.
859 RD->getIndirectPrimaryBases(Bases&: IndirectPrimaryBases);
860
861 // If the record has a dynamic base class, attempt to choose a primary base
862 // class. It is the first (in direct base class order) non-virtual dynamic
863 // base class, if one exists.
864 for (const auto &I : RD->bases()) {
865 // Ignore virtual bases.
866 if (I.isVirtual())
867 continue;
868
869 const CXXRecordDecl *Base = I.getType()->getAsCXXRecordDecl();
870
871 if (Base->isDynamicClass()) {
872 // We found it.
873 PrimaryBase = Base;
874 PrimaryBaseIsVirtual = false;
875 return;
876 }
877 }
878
879 // Under the Itanium ABI, if there is no non-virtual primary base class,
880 // try to compute the primary virtual base. The primary virtual base is
881 // the first nearly empty virtual base that is not an indirect primary
882 // virtual base class, if one exists.
883 if (RD->getNumVBases() != 0) {
884 SelectPrimaryVBase(RD);
885 if (PrimaryBase)
886 return;
887 }
888
889 // Otherwise, it is the first indirect primary base class, if one exists.
890 if (FirstNearlyEmptyVBase) {
891 PrimaryBase = FirstNearlyEmptyVBase;
892 PrimaryBaseIsVirtual = true;
893 return;
894 }
895
896 assert(!PrimaryBase && "Should not get here with a primary base!");
897}
898
899BaseSubobjectInfo *ItaniumRecordLayoutBuilder::ComputeBaseSubobjectInfo(
900 const CXXRecordDecl *RD, bool IsVirtual, BaseSubobjectInfo *Derived) {
901 BaseSubobjectInfo *Info;
902
903 if (IsVirtual) {
904 // Check if we already have info about this virtual base.
905 BaseSubobjectInfo *&InfoSlot = VirtualBaseInfo[RD];
906 if (InfoSlot) {
907 assert(InfoSlot->Class == RD && "Wrong class for virtual base info!");
908 return InfoSlot;
909 }
910
911 // We don't, create it.
912 InfoSlot = new (BaseSubobjectInfoAllocator.Allocate()) BaseSubobjectInfo;
913 Info = InfoSlot;
914 } else {
915 Info = new (BaseSubobjectInfoAllocator.Allocate()) BaseSubobjectInfo;
916 }
917
918 Info->Class = RD;
919 Info->IsVirtual = IsVirtual;
920 Info->Derived = nullptr;
921 Info->PrimaryVirtualBaseInfo = nullptr;
922
923 const CXXRecordDecl *PrimaryVirtualBase = nullptr;
924 BaseSubobjectInfo *PrimaryVirtualBaseInfo = nullptr;
925
926 // Check if this base has a primary virtual base.
927 if (RD->getNumVBases()) {
928 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: RD);
929 if (Layout.isPrimaryBaseVirtual()) {
930 // This base does have a primary virtual base.
931 PrimaryVirtualBase = Layout.getPrimaryBase();
932 assert(PrimaryVirtualBase && "Didn't have a primary virtual base!");
933
934 // Now check if we have base subobject info about this primary base.
935 PrimaryVirtualBaseInfo = VirtualBaseInfo.lookup(Val: PrimaryVirtualBase);
936
937 if (PrimaryVirtualBaseInfo) {
938 if (PrimaryVirtualBaseInfo->Derived) {
939 // We did have info about this primary base, and it turns out that it
940 // has already been claimed as a primary virtual base for another
941 // base.
942 PrimaryVirtualBase = nullptr;
943 } else {
944 // We can claim this base as our primary base.
945 Info->PrimaryVirtualBaseInfo = PrimaryVirtualBaseInfo;
946 PrimaryVirtualBaseInfo->Derived = Info;
947 }
948 }
949 }
950 }
951
952 // Now go through all direct bases.
953 for (const auto &I : RD->bases()) {
954 bool IsVirtual = I.isVirtual();
955
956 const CXXRecordDecl *BaseDecl = I.getType()->getAsCXXRecordDecl();
957
958 Info->Bases.push_back(Elt: ComputeBaseSubobjectInfo(RD: BaseDecl, IsVirtual, Derived: Info));
959 }
960
961 if (PrimaryVirtualBase && !PrimaryVirtualBaseInfo) {
962 // Traversing the bases must have created the base info for our primary
963 // virtual base.
964 PrimaryVirtualBaseInfo = VirtualBaseInfo.lookup(Val: PrimaryVirtualBase);
965 assert(PrimaryVirtualBaseInfo &&
966 "Did not create a primary virtual base!");
967
968 // Claim the primary virtual base as our primary virtual base.
969 Info->PrimaryVirtualBaseInfo = PrimaryVirtualBaseInfo;
970 PrimaryVirtualBaseInfo->Derived = Info;
971 }
972
973 return Info;
974}
975
976void ItaniumRecordLayoutBuilder::ComputeBaseSubobjectInfo(
977 const CXXRecordDecl *RD) {
978 for (const auto &I : RD->bases()) {
979 bool IsVirtual = I.isVirtual();
980
981 const CXXRecordDecl *BaseDecl = I.getType()->getAsCXXRecordDecl();
982
983 // Compute the base subobject info for this base.
984 BaseSubobjectInfo *Info = ComputeBaseSubobjectInfo(RD: BaseDecl, IsVirtual,
985 Derived: nullptr);
986
987 if (IsVirtual) {
988 // ComputeBaseInfo has already added this base for us.
989 assert(VirtualBaseInfo.count(BaseDecl) &&
990 "Did not add virtual base!");
991 } else {
992 // Add the base info to the map of non-virtual bases.
993 assert(!NonVirtualBaseInfo.count(BaseDecl) &&
994 "Non-virtual base already exists!");
995 NonVirtualBaseInfo.insert(KV: std::make_pair(x&: BaseDecl, y&: Info));
996 }
997 }
998}
999
1000void ItaniumRecordLayoutBuilder::EnsureVTablePointerAlignment(
1001 CharUnits UnpackedBaseAlign) {
1002 CharUnits BaseAlign = Packed ? CharUnits::One() : UnpackedBaseAlign;
1003
1004 // The maximum field alignment overrides base align.
1005 if (!MaxFieldAlignment.isZero()) {
1006 BaseAlign = std::min(a: BaseAlign, b: MaxFieldAlignment);
1007 UnpackedBaseAlign = std::min(a: UnpackedBaseAlign, b: MaxFieldAlignment);
1008 }
1009
1010 // Round up the current record size to pointer alignment.
1011 setSize(getSize().alignTo(Align: BaseAlign));
1012
1013 // Update the alignment.
1014 UpdateAlignment(NewAlignment: BaseAlign, UnpackedNewAlignment: UnpackedBaseAlign, PreferredAlignment: BaseAlign);
1015}
1016
1017void ItaniumRecordLayoutBuilder::LayoutNonVirtualBases(
1018 const CXXRecordDecl *RD) {
1019 // Then, determine the primary base class.
1020 DeterminePrimaryBase(RD);
1021
1022 // Compute base subobject info.
1023 ComputeBaseSubobjectInfo(RD);
1024
1025 // If we have a primary base class, lay it out.
1026 if (PrimaryBase) {
1027 if (PrimaryBaseIsVirtual) {
1028 // If the primary virtual base was a primary virtual base of some other
1029 // base class we'll have to steal it.
1030 BaseSubobjectInfo *PrimaryBaseInfo = VirtualBaseInfo.lookup(Val: PrimaryBase);
1031 PrimaryBaseInfo->Derived = nullptr;
1032
1033 // We have a virtual primary base, insert it as an indirect primary base.
1034 IndirectPrimaryBases.insert(Ptr: PrimaryBase);
1035
1036 assert(!VisitedVirtualBases.count(PrimaryBase) &&
1037 "vbase already visited!");
1038 VisitedVirtualBases.insert(Ptr: PrimaryBase);
1039
1040 LayoutVirtualBase(Base: PrimaryBaseInfo);
1041 } else {
1042 BaseSubobjectInfo *PrimaryBaseInfo =
1043 NonVirtualBaseInfo.lookup(Val: PrimaryBase);
1044 assert(PrimaryBaseInfo &&
1045 "Did not find base info for non-virtual primary base!");
1046
1047 LayoutNonVirtualBase(Base: PrimaryBaseInfo);
1048 }
1049
1050 // If this class needs a vtable/vf-table and didn't get one from a
1051 // primary base, add it in now.
1052 } else if (RD->isDynamicClass()) {
1053 assert(DataSize == 0 && "Vtable pointer must be at offset zero!");
1054 CharUnits PtrWidth = Context.toCharUnitsFromBits(
1055 BitSize: Context.getTargetInfo().getPointerWidth(AddrSpace: LangAS::Default));
1056 CharUnits PtrAlign = Context.toCharUnitsFromBits(
1057 BitSize: Context.getTargetInfo().getPointerAlign(AddrSpace: LangAS::Default));
1058 EnsureVTablePointerAlignment(UnpackedBaseAlign: PtrAlign);
1059 HasOwnVFPtr = true;
1060
1061 assert(!IsUnion && "Unions cannot be dynamic classes.");
1062 HandledFirstNonOverlappingEmptyField = true;
1063
1064 setSize(getSize() + PtrWidth);
1065 setDataSize(getSize());
1066 }
1067
1068 // Now lay out the non-virtual bases.
1069 for (const auto &I : RD->bases()) {
1070
1071 // Ignore virtual bases.
1072 if (I.isVirtual())
1073 continue;
1074
1075 const CXXRecordDecl *BaseDecl = I.getType()->getAsCXXRecordDecl();
1076
1077 // Skip the primary base, because we've already laid it out. The
1078 // !PrimaryBaseIsVirtual check is required because we might have a
1079 // non-virtual base of the same type as a primary virtual base.
1080 if (BaseDecl == PrimaryBase && !PrimaryBaseIsVirtual)
1081 continue;
1082
1083 // Lay out the base.
1084 BaseSubobjectInfo *BaseInfo = NonVirtualBaseInfo.lookup(Val: BaseDecl);
1085 assert(BaseInfo && "Did not find base info for non-virtual base!");
1086
1087 LayoutNonVirtualBase(Base: BaseInfo);
1088 }
1089}
1090
1091void ItaniumRecordLayoutBuilder::LayoutNonVirtualBase(
1092 const BaseSubobjectInfo *Base) {
1093 // Layout the base.
1094 CharUnits Offset = LayoutBase(Base);
1095
1096 // Add its base class offset.
1097 assert(!Bases.count(Base->Class) && "base offset already exists!");
1098 Bases.insert(KV: std::make_pair(x: Base->Class, y&: Offset));
1099
1100 AddPrimaryVirtualBaseOffsets(Info: Base, Offset);
1101}
1102
1103void ItaniumRecordLayoutBuilder::AddPrimaryVirtualBaseOffsets(
1104 const BaseSubobjectInfo *Info, CharUnits Offset) {
1105 // This base isn't interesting, it has no virtual bases.
1106 if (!Info->Class->getNumVBases())
1107 return;
1108
1109 // First, check if we have a virtual primary base to add offsets for.
1110 if (Info->PrimaryVirtualBaseInfo) {
1111 assert(Info->PrimaryVirtualBaseInfo->IsVirtual &&
1112 "Primary virtual base is not virtual!");
1113 if (Info->PrimaryVirtualBaseInfo->Derived == Info) {
1114 // Add the offset.
1115 assert(!VBases.count(Info->PrimaryVirtualBaseInfo->Class) &&
1116 "primary vbase offset already exists!");
1117 VBases.insert(KV: std::make_pair(x&: Info->PrimaryVirtualBaseInfo->Class,
1118 y: ASTRecordLayout::VBaseInfo(Offset, false)));
1119
1120 // Traverse the primary virtual base.
1121 AddPrimaryVirtualBaseOffsets(Info: Info->PrimaryVirtualBaseInfo, Offset);
1122 }
1123 }
1124
1125 // Now go through all direct non-virtual bases.
1126 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: Info->Class);
1127 for (const BaseSubobjectInfo *Base : Info->Bases) {
1128 if (Base->IsVirtual)
1129 continue;
1130
1131 CharUnits BaseOffset = Offset + Layout.getBaseClassOffset(Base: Base->Class);
1132 AddPrimaryVirtualBaseOffsets(Info: Base, Offset: BaseOffset);
1133 }
1134}
1135
1136void ItaniumRecordLayoutBuilder::LayoutVirtualBases(
1137 const CXXRecordDecl *RD, const CXXRecordDecl *MostDerivedClass) {
1138 const CXXRecordDecl *PrimaryBase;
1139 bool PrimaryBaseIsVirtual;
1140
1141 if (MostDerivedClass == RD) {
1142 PrimaryBase = this->PrimaryBase;
1143 PrimaryBaseIsVirtual = this->PrimaryBaseIsVirtual;
1144 } else {
1145 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: RD);
1146 PrimaryBase = Layout.getPrimaryBase();
1147 PrimaryBaseIsVirtual = Layout.isPrimaryBaseVirtual();
1148 }
1149
1150 for (const CXXBaseSpecifier &Base : RD->bases()) {
1151 assert(!Base.getType()->isDependentType() &&
1152 "Cannot layout class with dependent bases.");
1153
1154 const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
1155
1156 if (Base.isVirtual()) {
1157 if (PrimaryBase != BaseDecl || !PrimaryBaseIsVirtual) {
1158 bool IndirectPrimaryBase = IndirectPrimaryBases.count(Ptr: BaseDecl);
1159
1160 // Only lay out the virtual base if it's not an indirect primary base.
1161 if (!IndirectPrimaryBase) {
1162 // Only visit virtual bases once.
1163 if (!VisitedVirtualBases.insert(Ptr: BaseDecl).second)
1164 continue;
1165
1166 const BaseSubobjectInfo *BaseInfo = VirtualBaseInfo.lookup(Val: BaseDecl);
1167 assert(BaseInfo && "Did not find virtual base info!");
1168 LayoutVirtualBase(Base: BaseInfo);
1169 }
1170 }
1171 }
1172
1173 if (!BaseDecl->getNumVBases()) {
1174 // This base isn't interesting since it doesn't have any virtual bases.
1175 continue;
1176 }
1177
1178 LayoutVirtualBases(RD: BaseDecl, MostDerivedClass);
1179 }
1180}
1181
1182void ItaniumRecordLayoutBuilder::LayoutVirtualBase(
1183 const BaseSubobjectInfo *Base) {
1184 assert(!Base->Derived && "Trying to lay out a primary virtual base!");
1185
1186 // Layout the base.
1187 CharUnits Offset = LayoutBase(Base);
1188
1189 // Add its base class offset.
1190 assert(!VBases.count(Base->Class) && "vbase offset already exists!");
1191 VBases.insert(KV: std::make_pair(x: Base->Class,
1192 y: ASTRecordLayout::VBaseInfo(Offset, false)));
1193
1194 AddPrimaryVirtualBaseOffsets(Info: Base, Offset);
1195}
1196
1197CharUnits
1198ItaniumRecordLayoutBuilder::LayoutBase(const BaseSubobjectInfo *Base) {
1199 assert(!IsUnion && "Unions cannot have base classes.");
1200
1201 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: Base->Class);
1202 CharUnits Offset;
1203
1204 // Query the external layout to see if it provides an offset.
1205 bool HasExternalLayout = false;
1206 if (UseExternalLayout) {
1207 if (Base->IsVirtual)
1208 HasExternalLayout = External.getExternalVBaseOffset(RD: Base->Class, BaseOffset&: Offset);
1209 else
1210 HasExternalLayout = External.getExternalNVBaseOffset(RD: Base->Class, BaseOffset&: Offset);
1211 }
1212
1213 auto getBaseOrPreferredBaseAlignFromUnpacked = [&](CharUnits UnpackedAlign) {
1214 // Clang <= 6 incorrectly applied the 'packed' attribute to base classes.
1215 // Per GCC's documentation, it only applies to non-static data members.
1216 return (Packed && (Context.getLangOpts().isCompatibleWith(
1217 Version: LangOptions::ClangABI::Ver6) ||
1218 Context.getTargetInfo().getTriple().isPS() ||
1219 Context.getTargetInfo().getTriple().isOSAIX()))
1220 ? CharUnits::One()
1221 : UnpackedAlign;
1222 };
1223
1224 CharUnits UnpackedBaseAlign = Layout.getNonVirtualAlignment();
1225 CharUnits UnpackedPreferredBaseAlign = Layout.getPreferredNVAlignment();
1226 CharUnits BaseAlign =
1227 getBaseOrPreferredBaseAlignFromUnpacked(UnpackedBaseAlign);
1228 CharUnits PreferredBaseAlign =
1229 getBaseOrPreferredBaseAlignFromUnpacked(UnpackedPreferredBaseAlign);
1230
1231 const bool DefaultsToAIXPowerAlignment =
1232 Context.getTargetInfo().defaultsToAIXPowerAlignment();
1233 if (DefaultsToAIXPowerAlignment) {
1234 // AIX `power` alignment does not apply the preferred alignment for
1235 // non-union classes if the source of the alignment (the current base in
1236 // this context) follows introduction of the first subobject with
1237 // exclusively allocated space or zero-extent array.
1238 if (!Base->Class->isEmpty() && !HandledFirstNonOverlappingEmptyField) {
1239 // By handling a base class that is not empty, we're handling the
1240 // "first (inherited) member".
1241 HandledFirstNonOverlappingEmptyField = true;
1242 } else if (!IsNaturalAlign) {
1243 UnpackedPreferredBaseAlign = UnpackedBaseAlign;
1244 PreferredBaseAlign = BaseAlign;
1245 }
1246 }
1247
1248 CharUnits UnpackedAlignTo = !DefaultsToAIXPowerAlignment
1249 ? UnpackedBaseAlign
1250 : UnpackedPreferredBaseAlign;
1251 // If we have an empty base class, try to place it at offset 0.
1252 if (Base->Class->isEmpty() &&
1253 (!HasExternalLayout || Offset == CharUnits::Zero()) &&
1254 EmptySubobjects->CanPlaceBaseAtOffset(Info: Base, Offset: CharUnits::Zero())) {
1255 setSize(std::max(a: getSize(), b: Layout.getSize()));
1256 // On PS4/PS5, don't update the alignment, to preserve compatibility.
1257 if (!Context.getTargetInfo().getTriple().isPS())
1258 UpdateAlignment(NewAlignment: BaseAlign, UnpackedNewAlignment: UnpackedAlignTo, PreferredAlignment: PreferredBaseAlign);
1259
1260 return CharUnits::Zero();
1261 }
1262
1263 // The maximum field alignment overrides the base align/(AIX-only) preferred
1264 // base align.
1265 if (!MaxFieldAlignment.isZero()) {
1266 BaseAlign = std::min(a: BaseAlign, b: MaxFieldAlignment);
1267 PreferredBaseAlign = std::min(a: PreferredBaseAlign, b: MaxFieldAlignment);
1268 UnpackedAlignTo = std::min(a: UnpackedAlignTo, b: MaxFieldAlignment);
1269 }
1270
1271 CharUnits AlignTo =
1272 !DefaultsToAIXPowerAlignment ? BaseAlign : PreferredBaseAlign;
1273 if (!HasExternalLayout) {
1274 // Round up the current record size to the base's alignment boundary.
1275 Offset = getDataSize().alignTo(Align: AlignTo);
1276
1277 // Try to place the base.
1278 while (!EmptySubobjects->CanPlaceBaseAtOffset(Info: Base, Offset))
1279 Offset += AlignTo;
1280 } else {
1281 bool Allowed = EmptySubobjects->CanPlaceBaseAtOffset(Info: Base, Offset);
1282 (void)Allowed;
1283 assert(Allowed && "Base subobject externally placed at overlapping offset");
1284
1285 if (InferAlignment && Offset < getDataSize().alignTo(Align: AlignTo)) {
1286 // The externally-supplied base offset is before the base offset we
1287 // computed. Assume that the structure is packed.
1288 Alignment = CharUnits::One();
1289 InferAlignment = false;
1290 }
1291 }
1292
1293 if (!Base->Class->isEmpty()) {
1294 // Update the data size.
1295 setDataSize(Offset + Layout.getNonVirtualSize());
1296
1297 setSize(std::max(a: getSize(), b: getDataSize()));
1298 } else
1299 setSize(std::max(a: getSize(), b: Offset + Layout.getSize()));
1300
1301 // Remember max struct/class alignment.
1302 UnadjustedAlignment = std::max(a: UnadjustedAlignment, b: BaseAlign);
1303 UpdateAlignment(NewAlignment: BaseAlign, UnpackedNewAlignment: UnpackedAlignTo, PreferredAlignment: PreferredBaseAlign);
1304
1305 return Offset;
1306}
1307
1308void ItaniumRecordLayoutBuilder::InitializeLayout(const Decl *D) {
1309 if (const RecordDecl *RD = dyn_cast<RecordDecl>(Val: D)) {
1310 IsUnion = RD->isUnion();
1311 IsMsStruct = RD->isMsStruct(C: Context);
1312 }
1313
1314 Packed = D->hasAttr<PackedAttr>();
1315
1316 // Honor the default struct packing maximum alignment flag.
1317 if (unsigned DefaultMaxFieldAlignment = Context.getLangOpts().PackStruct) {
1318 MaxFieldAlignment = CharUnits::fromQuantity(Quantity: DefaultMaxFieldAlignment);
1319 }
1320
1321 // mac68k alignment supersedes maximum field alignment and attribute aligned,
1322 // and forces all structures to have 2-byte alignment. The IBM docs on it
1323 // allude to additional (more complicated) semantics, especially with regard
1324 // to bit-fields, but gcc appears not to follow that.
1325 if (D->hasAttr<AlignMac68kAttr>()) {
1326 assert(
1327 !D->hasAttr<AlignNaturalAttr>() &&
1328 "Having both mac68k and natural alignment on a decl is not allowed.");
1329 IsMac68kAlign = true;
1330 MaxFieldAlignment = CharUnits::fromQuantity(Quantity: 2);
1331 Alignment = CharUnits::fromQuantity(Quantity: 2);
1332 PreferredAlignment = CharUnits::fromQuantity(Quantity: 2);
1333 } else {
1334 if (D->hasAttr<AlignNaturalAttr>())
1335 IsNaturalAlign = true;
1336
1337 if (const MaxFieldAlignmentAttr *MFAA = D->getAttr<MaxFieldAlignmentAttr>())
1338 MaxFieldAlignment = Context.toCharUnitsFromBits(BitSize: MFAA->getAlignment());
1339
1340 if (unsigned MaxAlign = D->getMaxAlignment())
1341 UpdateAlignment(NewAlignment: Context.toCharUnitsFromBits(BitSize: MaxAlign));
1342 }
1343
1344 HandledFirstNonOverlappingEmptyField =
1345 !Context.getTargetInfo().defaultsToAIXPowerAlignment() || IsNaturalAlign;
1346
1347 // If there is an external AST source, ask it for the various offsets.
1348 if (const RecordDecl *RD = dyn_cast<RecordDecl>(Val: D))
1349 if (ExternalASTSource *Source = Context.getExternalSource()) {
1350 UseExternalLayout = Source->layoutRecordType(
1351 Record: RD, Size&: External.Size, Alignment&: External.Align, FieldOffsets&: External.FieldOffsets,
1352 BaseOffsets&: External.BaseOffsets, VirtualBaseOffsets&: External.VirtualBaseOffsets);
1353
1354 // Update based on external alignment.
1355 if (UseExternalLayout) {
1356 if (External.Align > 0) {
1357 Alignment = Context.toCharUnitsFromBits(BitSize: External.Align);
1358 PreferredAlignment = Context.toCharUnitsFromBits(BitSize: External.Align);
1359 } else {
1360 // The external source didn't have alignment information; infer it.
1361 InferAlignment = true;
1362 }
1363 }
1364 }
1365}
1366
1367void ItaniumRecordLayoutBuilder::Layout(const RecordDecl *D) {
1368 InitializeLayout(D);
1369 LayoutFields(D);
1370
1371 // Finally, round the size of the total struct up to the alignment of the
1372 // struct itself.
1373 FinishLayout(D);
1374}
1375
1376void ItaniumRecordLayoutBuilder::Layout(const CXXRecordDecl *RD) {
1377 InitializeLayout(D: RD);
1378
1379 // Lay out the vtable and the non-virtual bases.
1380 LayoutNonVirtualBases(RD);
1381
1382 LayoutFields(D: RD);
1383
1384 NonVirtualSize = Context.toCharUnitsFromBits(
1385 BitSize: llvm::alignTo(Value: getSizeInBits(), Align: Context.getTargetInfo().getCharAlign()));
1386 NonVirtualAlignment = Alignment;
1387 PreferredNVAlignment = PreferredAlignment;
1388
1389 // Lay out the virtual bases and add the primary virtual base offsets.
1390 LayoutVirtualBases(RD, MostDerivedClass: RD);
1391
1392 // Finally, round the size of the total struct up to the alignment
1393 // of the struct itself.
1394 FinishLayout(D: RD);
1395
1396#ifndef NDEBUG
1397 // Check that we have base offsets for all bases.
1398 for (const CXXBaseSpecifier &Base : RD->bases()) {
1399 if (Base.isVirtual())
1400 continue;
1401
1402 const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
1403
1404 assert(Bases.count(BaseDecl) && "Did not find base offset!");
1405 }
1406
1407 // And all virtual bases.
1408 for (const CXXBaseSpecifier &Base : RD->vbases()) {
1409 const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
1410
1411 assert(VBases.count(BaseDecl) && "Did not find base offset!");
1412 }
1413#endif
1414}
1415
1416void ItaniumRecordLayoutBuilder::Layout(const ObjCInterfaceDecl *D) {
1417 if (ObjCInterfaceDecl *SD = D->getSuperClass()) {
1418 const ASTRecordLayout &SL = Context.getASTObjCInterfaceLayout(D: SD);
1419
1420 UpdateAlignment(NewAlignment: SL.getAlignment());
1421
1422 // We start laying out ivars not at the end of the superclass
1423 // structure, but at the next byte following the last field.
1424 setDataSize(SL.getDataSize());
1425 setSize(getDataSize());
1426 }
1427
1428 InitializeLayout(D);
1429 // Layout each ivar sequentially.
1430 for (const ObjCIvarDecl *IVD = D->all_declared_ivar_begin(); IVD;
1431 IVD = IVD->getNextIvar())
1432 LayoutField(D: IVD, InsertExtraPadding: false);
1433
1434 // Finally, round the size of the total struct up to the alignment of the
1435 // struct itself.
1436 FinishLayout(D);
1437}
1438
1439void ItaniumRecordLayoutBuilder::LayoutFields(const RecordDecl *D) {
1440 // Layout each field, for now, just sequentially, respecting alignment. In
1441 // the future, this will need to be tweakable by targets.
1442 bool InsertExtraPadding = D->mayInsertExtraPadding(/*EmitRemark=*/true);
1443 bool HasFlexibleArrayMember = D->hasFlexibleArrayMember();
1444 for (auto I = D->field_begin(), End = D->field_end(); I != End; ++I) {
1445 LayoutField(D: *I, InsertExtraPadding: InsertExtraPadding &&
1446 (std::next(x: I) != End || !HasFlexibleArrayMember));
1447 }
1448}
1449
1450// Rounds the specified size to have it a multiple of the char size.
1451static uint64_t
1452roundUpSizeToCharAlignment(uint64_t Size,
1453 const ASTContext &Context) {
1454 uint64_t CharAlignment = Context.getTargetInfo().getCharAlign();
1455 return llvm::alignTo(Value: Size, Align: CharAlignment);
1456}
1457
1458void ItaniumRecordLayoutBuilder::LayoutWideBitField(uint64_t FieldSize,
1459 uint64_t StorageUnitSize,
1460 bool FieldPacked,
1461 const FieldDecl *D) {
1462 assert(Context.getLangOpts().CPlusPlus &&
1463 "Can only have wide bit-fields in C++!");
1464
1465 // Itanium C++ ABI 2.4:
1466 // If sizeof(T)*8 < n, let T' be the largest integral POD type with
1467 // sizeof(T')*8 <= n.
1468
1469 QualType IntegralPODTypes[] = {
1470 Context.UnsignedCharTy, Context.UnsignedShortTy,
1471 Context.UnsignedIntTy, Context.UnsignedLongTy,
1472 Context.UnsignedLongLongTy, Context.UnsignedInt128Ty,
1473 };
1474
1475 QualType Type;
1476 uint64_t MaxSize =
1477 Context.getTargetInfo().getLargestOverSizedBitfieldContainer();
1478 for (const QualType &QT : IntegralPODTypes) {
1479 uint64_t Size = Context.getTypeSize(T: QT);
1480
1481 if (Size > FieldSize || Size > MaxSize)
1482 break;
1483
1484 Type = QT;
1485 }
1486 assert(!Type.isNull() && "Did not find a type!");
1487
1488 CharUnits TypeAlign = Context.getTypeAlignInChars(T: Type);
1489
1490 // We're not going to use any of the unfilled bits in the last byte.
1491 UnfilledBitsInLastUnit = 0;
1492 LastBitfieldStorageUnitSize = 0;
1493
1494 uint64_t FieldOffset;
1495 uint64_t UnpaddedFieldOffset = getDataSizeInBits() - UnfilledBitsInLastUnit;
1496
1497 if (IsUnion) {
1498 uint64_t RoundedFieldSize = roundUpSizeToCharAlignment(Size: FieldSize,
1499 Context);
1500 setDataSize(std::max(a: getDataSizeInBits(), b: RoundedFieldSize));
1501 FieldOffset = 0;
1502 } else {
1503 // The bitfield is allocated starting at the next offset aligned
1504 // appropriately for T', with length n bits.
1505 FieldOffset = llvm::alignTo(Value: getDataSizeInBits(), Align: Context.toBits(CharSize: TypeAlign));
1506
1507 uint64_t NewSizeInBits = FieldOffset + FieldSize;
1508
1509 setDataSize(
1510 llvm::alignTo(Value: NewSizeInBits, Align: Context.getTargetInfo().getCharAlign()));
1511 UnfilledBitsInLastUnit = getDataSizeInBits() - NewSizeInBits;
1512 }
1513
1514 // Place this field at the current location.
1515 FieldOffsets.push_back(Elt: FieldOffset);
1516
1517 CheckFieldPadding(Offset: FieldOffset, UnpaddedOffset: UnpaddedFieldOffset, UnpackedOffset: FieldOffset,
1518 UnpackedAlign: Context.toBits(CharSize: TypeAlign), isPacked: FieldPacked, D);
1519
1520 // Update the size.
1521 setSize(std::max(a: getSizeInBits(), b: getDataSizeInBits()));
1522
1523 // Remember max struct/class alignment.
1524 UnadjustedAlignment = std::max(a: UnadjustedAlignment, b: TypeAlign);
1525 UpdateAlignment(NewAlignment: TypeAlign);
1526}
1527
1528static bool isAIXLayout(const ASTContext &Context) {
1529 return Context.getTargetInfo().getTriple().getOS() == llvm::Triple::AIX;
1530}
1531
1532void ItaniumRecordLayoutBuilder::LayoutBitField(const FieldDecl *D) {
1533 bool FieldPacked = Packed || D->hasAttr<PackedAttr>();
1534 uint64_t FieldSize = D->getBitWidthValue();
1535 TypeInfo FieldInfo = Context.getTypeInfo(T: D->getType());
1536 uint64_t StorageUnitSize = FieldInfo.Width;
1537 unsigned FieldAlign = FieldInfo.Align;
1538 bool AlignIsRequired = FieldInfo.isAlignRequired();
1539 unsigned char PaddingInLastUnit = 0;
1540
1541 // UnfilledBitsInLastUnit is the difference between the end of the
1542 // last allocated bitfield (i.e. the first bit offset available for
1543 // bitfields) and the end of the current data size in bits (i.e. the
1544 // first bit offset available for non-bitfields). The current data
1545 // size in bits is always a multiple of the char size; additionally,
1546 // for ms_struct records it's also a multiple of the
1547 // LastBitfieldStorageUnitSize (if set).
1548
1549 // The struct-layout algorithm is dictated by the platform ABI,
1550 // which in principle could use almost any rules it likes. In
1551 // practice, UNIXy targets tend to inherit the algorithm described
1552 // in the System V generic ABI. The basic bitfield layout rule in
1553 // System V is to place bitfields at the next available bit offset
1554 // where the entire bitfield would fit in an aligned storage unit of
1555 // the declared type; it's okay if an earlier or later non-bitfield
1556 // is allocated in the same storage unit. However, some targets
1557 // (those that !useBitFieldTypeAlignment(), e.g. ARM APCS) don't
1558 // require this storage unit to be aligned, and therefore always put
1559 // the bitfield at the next available bit offset.
1560
1561 // ms_struct basically requests a complete replacement of the
1562 // platform ABI's struct-layout algorithm, with the high-level goal
1563 // of duplicating MSVC's layout. For non-bitfields, this follows
1564 // the standard algorithm. The basic bitfield layout rule is to
1565 // allocate an entire unit of the bitfield's declared type
1566 // (e.g. 'unsigned long'), then parcel it up among successive
1567 // bitfields whose declared types have the same size, making a new
1568 // unit as soon as the last can no longer store the whole value.
1569 // Since it completely replaces the platform ABI's algorithm,
1570 // settings like !useBitFieldTypeAlignment() do not apply.
1571
1572 // A zero-width bitfield forces the use of a new storage unit for
1573 // later bitfields. In general, this occurs by rounding up the
1574 // current size of the struct as if the algorithm were about to
1575 // place a non-bitfield of the field's formal type. Usually this
1576 // does not change the alignment of the struct itself, but it does
1577 // on some targets (those that useZeroLengthBitfieldAlignment(),
1578 // e.g. ARM). In ms_struct layout, zero-width bitfields are
1579 // ignored unless they follow a non-zero-width bitfield.
1580
1581 // A field alignment restriction (e.g. from #pragma pack) or
1582 // specification (e.g. from __attribute__((aligned))) changes the
1583 // formal alignment of the field. For System V, this alters the
1584 // required alignment of the notional storage unit that must contain
1585 // the bitfield. For ms_struct, this only affects the placement of
1586 // new storage units. In both cases, the effect of #pragma pack is
1587 // ignored on zero-width bitfields.
1588
1589 // On System V, a packed field (e.g. from #pragma pack or
1590 // __attribute__((packed))) always uses the next available bit
1591 // offset.
1592
1593 // In an ms_struct struct, the alignment of a fundamental type is
1594 // always equal to its size. This is necessary in order to mimic
1595 // the i386 alignment rules on targets which might not fully align
1596 // all types (e.g. Darwin PPC32, where alignof(long long) == 4).
1597
1598 // First, some simple bookkeeping to perform for ms_struct structs.
1599 if (IsMsStruct) {
1600 // The field alignment for integer types is always the size.
1601 FieldAlign = StorageUnitSize;
1602
1603 // If the previous field was not a bitfield, or was a bitfield
1604 // with a different storage unit size, or if this field doesn't fit into
1605 // the current storage unit, we're done with that storage unit.
1606 if (LastBitfieldStorageUnitSize != StorageUnitSize ||
1607 UnfilledBitsInLastUnit < FieldSize) {
1608 // Also, ignore zero-length bitfields after non-bitfields.
1609 if (!LastBitfieldStorageUnitSize && !FieldSize)
1610 FieldAlign = 1;
1611
1612 PaddingInLastUnit = UnfilledBitsInLastUnit;
1613 UnfilledBitsInLastUnit = 0;
1614 LastBitfieldStorageUnitSize = 0;
1615 }
1616 }
1617
1618 if (isAIXLayout(Context)) {
1619 if (StorageUnitSize < Context.getTypeSize(T: Context.UnsignedIntTy)) {
1620 // On AIX, [bool, char, short] bitfields have the same alignment
1621 // as [unsigned].
1622 StorageUnitSize = Context.getTypeSize(T: Context.UnsignedIntTy);
1623 } else if (StorageUnitSize > Context.getTypeSize(T: Context.UnsignedIntTy) &&
1624 Context.getTargetInfo().getTriple().isArch32Bit() &&
1625 FieldSize <= 32) {
1626 // Under 32-bit compile mode, the bitcontainer is 32 bits if a single
1627 // long long bitfield has length no greater than 32 bits.
1628 StorageUnitSize = 32;
1629
1630 if (!AlignIsRequired)
1631 FieldAlign = 32;
1632 }
1633
1634 if (FieldAlign < StorageUnitSize) {
1635 // The bitfield alignment should always be greater than or equal to
1636 // bitcontainer size.
1637 FieldAlign = StorageUnitSize;
1638 }
1639 }
1640
1641 // If the field is wider than its declared type, it follows
1642 // different rules in all cases, except on AIX.
1643 // On AIX, wide bitfield follows the same rules as normal bitfield.
1644 if (FieldSize > StorageUnitSize && !isAIXLayout(Context)) {
1645 LayoutWideBitField(FieldSize, StorageUnitSize, FieldPacked, D);
1646 return;
1647 }
1648
1649 // Compute the next available bit offset.
1650 uint64_t FieldOffset =
1651 IsUnion ? 0 : (getDataSizeInBits() - UnfilledBitsInLastUnit);
1652
1653 // Handle targets that don't honor bitfield type alignment.
1654 if (!IsMsStruct && !Context.getTargetInfo().useBitFieldTypeAlignment()) {
1655 // Some such targets do honor it on zero-width bitfields.
1656 if (FieldSize == 0 &&
1657 Context.getTargetInfo().useZeroLengthBitfieldAlignment()) {
1658 // Some targets don't honor leading zero-width bitfield.
1659 if (!IsUnion && FieldOffset == 0 &&
1660 !Context.getTargetInfo().useLeadingZeroLengthBitfield())
1661 FieldAlign = 1;
1662 else {
1663 // The alignment to round up to is the max of the field's natural
1664 // alignment and a target-specific fixed value (sometimes zero).
1665 unsigned ZeroLengthBitfieldBoundary =
1666 Context.getTargetInfo().getZeroLengthBitfieldBoundary();
1667 FieldAlign = std::max(a: FieldAlign, b: ZeroLengthBitfieldBoundary);
1668 }
1669 // If that doesn't apply, just ignore the field alignment.
1670 } else {
1671 FieldAlign = 1;
1672 }
1673 }
1674
1675 // Remember the alignment we would have used if the field were not packed.
1676 unsigned UnpackedFieldAlign = FieldAlign;
1677
1678 // Ignore the field alignment if the field is packed unless it has zero-size.
1679 if (FieldPacked && FieldSize != 0)
1680 FieldAlign = 1;
1681
1682 // But, if there's an 'aligned' attribute on the field, honor that.
1683 unsigned ExplicitFieldAlign = D->getMaxAlignment();
1684 if (ExplicitFieldAlign) {
1685 FieldAlign = std::max(a: FieldAlign, b: ExplicitFieldAlign);
1686 UnpackedFieldAlign = std::max(a: UnpackedFieldAlign, b: ExplicitFieldAlign);
1687 }
1688
1689 // But, if there's a #pragma pack in play, that takes precedent over
1690 // even the 'aligned' attribute, for non-zero-width bitfields.
1691 unsigned MaxFieldAlignmentInBits = Context.toBits(CharSize: MaxFieldAlignment);
1692 if (!MaxFieldAlignment.isZero() && FieldSize) {
1693 UnpackedFieldAlign = std::min(a: UnpackedFieldAlign, b: MaxFieldAlignmentInBits);
1694 if (FieldPacked)
1695 FieldAlign = UnpackedFieldAlign;
1696 else
1697 FieldAlign = std::min(a: FieldAlign, b: MaxFieldAlignmentInBits);
1698 }
1699
1700 // But, ms_struct just ignores all of that in unions, even explicit
1701 // alignment attributes.
1702 if (IsMsStruct && IsUnion) {
1703 FieldAlign = UnpackedFieldAlign = 1;
1704 }
1705
1706 // For purposes of diagnostics, we're going to simultaneously
1707 // compute the field offsets that we would have used if we weren't
1708 // adding any alignment padding or if the field weren't packed.
1709 uint64_t UnpaddedFieldOffset = FieldOffset - PaddingInLastUnit;
1710 uint64_t UnpackedFieldOffset = FieldOffset;
1711
1712 // Check if we need to add padding to fit the bitfield within an
1713 // allocation unit with the right size and alignment. The rules are
1714 // somewhat different here for ms_struct structs.
1715 if (IsMsStruct) {
1716 // If it's not a zero-width bitfield, and we can fit the bitfield
1717 // into the active storage unit (and we haven't already decided to
1718 // start a new storage unit), just do so, regardless of any other
1719 // other consideration. Otherwise, round up to the right alignment.
1720 if (FieldSize == 0 || FieldSize > UnfilledBitsInLastUnit) {
1721 FieldOffset = llvm::alignTo(Value: FieldOffset, Align: FieldAlign);
1722 UnpackedFieldOffset =
1723 llvm::alignTo(Value: UnpackedFieldOffset, Align: UnpackedFieldAlign);
1724 UnfilledBitsInLastUnit = 0;
1725 }
1726
1727 } else {
1728 // #pragma pack, with any value, suppresses the insertion of padding.
1729 bool AllowPadding = MaxFieldAlignment.isZero();
1730
1731 // Compute the real offset.
1732 if (FieldSize == 0 ||
1733 (AllowPadding &&
1734 (FieldOffset & (FieldAlign - 1)) + FieldSize > StorageUnitSize)) {
1735 FieldOffset = llvm::alignTo(Value: FieldOffset, Align: FieldAlign);
1736 } else if (ExplicitFieldAlign &&
1737 (MaxFieldAlignmentInBits == 0 ||
1738 ExplicitFieldAlign <= MaxFieldAlignmentInBits) &&
1739 Context.getTargetInfo().useExplicitBitFieldAlignment()) {
1740 // TODO: figure it out what needs to be done on targets that don't honor
1741 // bit-field type alignment like ARM APCS ABI.
1742 FieldOffset = llvm::alignTo(Value: FieldOffset, Align: ExplicitFieldAlign);
1743 }
1744
1745 // Repeat the computation for diagnostic purposes.
1746 if (FieldSize == 0 ||
1747 (AllowPadding &&
1748 (UnpackedFieldOffset & (UnpackedFieldAlign - 1)) + FieldSize >
1749 StorageUnitSize))
1750 UnpackedFieldOffset =
1751 llvm::alignTo(Value: UnpackedFieldOffset, Align: UnpackedFieldAlign);
1752 else if (ExplicitFieldAlign &&
1753 (MaxFieldAlignmentInBits == 0 ||
1754 ExplicitFieldAlign <= MaxFieldAlignmentInBits) &&
1755 Context.getTargetInfo().useExplicitBitFieldAlignment())
1756 UnpackedFieldOffset =
1757 llvm::alignTo(Value: UnpackedFieldOffset, Align: ExplicitFieldAlign);
1758 }
1759
1760 // If we're using external layout, give the external layout a chance
1761 // to override this information.
1762 if (UseExternalLayout)
1763 FieldOffset = updateExternalFieldOffset(Field: D, ComputedOffset: FieldOffset);
1764
1765 // Okay, place the bitfield at the calculated offset.
1766 FieldOffsets.push_back(Elt: FieldOffset);
1767
1768 // Bookkeeping:
1769
1770 // Anonymous members don't affect the overall record alignment,
1771 // except on targets where they do.
1772 if (!IsMsStruct &&
1773 !Context.getTargetInfo().useZeroLengthBitfieldAlignment() &&
1774 !D->getIdentifier())
1775 FieldAlign = UnpackedFieldAlign = 1;
1776
1777 // On AIX, zero-width bitfields pad out to the natural alignment boundary,
1778 // but do not increase the alignment greater than the MaxFieldAlignment, or 1
1779 // if packed.
1780 if (isAIXLayout(Context) && !FieldSize) {
1781 if (FieldPacked)
1782 FieldAlign = 1;
1783 if (!MaxFieldAlignment.isZero()) {
1784 UnpackedFieldAlign =
1785 std::min(a: UnpackedFieldAlign, b: MaxFieldAlignmentInBits);
1786 FieldAlign = std::min(a: FieldAlign, b: MaxFieldAlignmentInBits);
1787 }
1788 }
1789
1790 // Diagnose differences in layout due to padding or packing.
1791 if (!UseExternalLayout)
1792 CheckFieldPadding(Offset: FieldOffset, UnpaddedOffset: UnpaddedFieldOffset, UnpackedOffset: UnpackedFieldOffset,
1793 UnpackedAlign: UnpackedFieldAlign, isPacked: FieldPacked, D);
1794
1795 // Update DataSize to include the last byte containing (part of) the bitfield.
1796
1797 // For unions, this is just a max operation, as usual.
1798 if (IsUnion) {
1799 // For ms_struct, allocate the entire storage unit --- unless this
1800 // is a zero-width bitfield, in which case just use a size of 1.
1801 uint64_t RoundedFieldSize;
1802 if (IsMsStruct) {
1803 RoundedFieldSize = (FieldSize ? StorageUnitSize
1804 : Context.getTargetInfo().getCharWidth());
1805
1806 // Otherwise, allocate just the number of bytes required to store
1807 // the bitfield.
1808 } else {
1809 RoundedFieldSize = roundUpSizeToCharAlignment(Size: FieldSize, Context);
1810 }
1811 setDataSize(std::max(a: getDataSizeInBits(), b: RoundedFieldSize));
1812
1813 // For non-zero-width bitfields in ms_struct structs, allocate a new
1814 // storage unit if necessary.
1815 } else if (IsMsStruct && FieldSize) {
1816 // We should have cleared UnfilledBitsInLastUnit in every case
1817 // where we changed storage units.
1818 if (!UnfilledBitsInLastUnit) {
1819 setDataSize(FieldOffset + StorageUnitSize);
1820 UnfilledBitsInLastUnit = StorageUnitSize;
1821 }
1822 UnfilledBitsInLastUnit -= FieldSize;
1823 LastBitfieldStorageUnitSize = StorageUnitSize;
1824
1825 // Otherwise, bump the data size up to include the bitfield,
1826 // including padding up to char alignment, and then remember how
1827 // bits we didn't use.
1828 } else {
1829 uint64_t NewSizeInBits = FieldOffset + FieldSize;
1830 uint64_t CharAlignment = Context.getTargetInfo().getCharAlign();
1831 setDataSize(llvm::alignTo(Value: NewSizeInBits, Align: CharAlignment));
1832 UnfilledBitsInLastUnit = getDataSizeInBits() - NewSizeInBits;
1833
1834 // The only time we can get here for an ms_struct is if this is a
1835 // zero-width bitfield, which doesn't count as anything for the
1836 // purposes of unfilled bits.
1837 LastBitfieldStorageUnitSize = 0;
1838 }
1839
1840 // Update the size.
1841 setSize(std::max(a: getSizeInBits(), b: getDataSizeInBits()));
1842
1843 // Remember max struct/class alignment.
1844 UnadjustedAlignment =
1845 std::max(a: UnadjustedAlignment, b: Context.toCharUnitsFromBits(BitSize: FieldAlign));
1846 UpdateAlignment(NewAlignment: Context.toCharUnitsFromBits(BitSize: FieldAlign),
1847 UnpackedNewAlignment: Context.toCharUnitsFromBits(BitSize: UnpackedFieldAlign));
1848}
1849
1850void ItaniumRecordLayoutBuilder::LayoutField(const FieldDecl *D,
1851 bool InsertExtraPadding) {
1852 auto *FieldClass = D->getType()->getAsCXXRecordDecl();
1853 bool IsOverlappingEmptyField =
1854 D->isPotentiallyOverlapping() && FieldClass->isEmpty();
1855
1856 CharUnits FieldOffset =
1857 (IsUnion || IsOverlappingEmptyField) ? CharUnits::Zero() : getDataSize();
1858
1859 const bool DefaultsToAIXPowerAlignment =
1860 Context.getTargetInfo().defaultsToAIXPowerAlignment();
1861 bool FoundFirstNonOverlappingEmptyFieldForAIX = false;
1862 if (DefaultsToAIXPowerAlignment && !HandledFirstNonOverlappingEmptyField) {
1863 assert(FieldOffset == CharUnits::Zero() &&
1864 "The first non-overlapping empty field should have been handled.");
1865
1866 if (!IsOverlappingEmptyField) {
1867 FoundFirstNonOverlappingEmptyFieldForAIX = true;
1868
1869 // We're going to handle the "first member" based on
1870 // `FoundFirstNonOverlappingEmptyFieldForAIX` during the current
1871 // invocation of this function; record it as handled for future
1872 // invocations (except for unions, because the current field does not
1873 // represent all "firsts").
1874 HandledFirstNonOverlappingEmptyField = !IsUnion;
1875 }
1876 }
1877
1878 if (D->isBitField()) {
1879 LayoutBitField(D);
1880 return;
1881 }
1882
1883 uint64_t UnpaddedFieldOffset = getDataSizeInBits() - UnfilledBitsInLastUnit;
1884 // Reset the unfilled bits.
1885 UnfilledBitsInLastUnit = 0;
1886 LastBitfieldStorageUnitSize = 0;
1887
1888 llvm::Triple Target = Context.getTargetInfo().getTriple();
1889
1890 AlignRequirementKind AlignRequirement = AlignRequirementKind::None;
1891 CharUnits FieldSize;
1892 CharUnits FieldAlign;
1893 // The amount of this class's dsize occupied by the field.
1894 // This is equal to FieldSize unless we're permitted to pack
1895 // into the field's tail padding.
1896 CharUnits EffectiveFieldSize;
1897
1898 auto setDeclInfo = [&](bool IsIncompleteArrayType) {
1899 auto TI = Context.getTypeInfoInChars(T: D->getType());
1900 FieldAlign = TI.Align;
1901 // Flexible array members don't have any size, but they have to be
1902 // aligned appropriately for their element type.
1903 EffectiveFieldSize = FieldSize =
1904 IsIncompleteArrayType ? CharUnits::Zero() : TI.Width;
1905 AlignRequirement = TI.AlignRequirement;
1906 };
1907
1908 if (D->getType()->isIncompleteArrayType()) {
1909 setDeclInfo(true /* IsIncompleteArrayType */);
1910 } else {
1911 setDeclInfo(false /* IsIncompleteArrayType */);
1912
1913 // A potentially-overlapping field occupies its dsize or nvsize, whichever
1914 // is larger.
1915 if (D->isPotentiallyOverlapping()) {
1916 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: FieldClass);
1917 EffectiveFieldSize =
1918 std::max(a: Layout.getNonVirtualSize(), b: Layout.getDataSize());
1919 }
1920
1921 if (IsMsStruct) {
1922 // If MS bitfield layout is required, figure out what type is being
1923 // laid out and align the field to the width of that type.
1924
1925 // Resolve all typedefs down to their base type and round up the field
1926 // alignment if necessary.
1927 QualType T = Context.getBaseElementType(QT: D->getType());
1928 if (const BuiltinType *BTy = T->getAs<BuiltinType>()) {
1929 CharUnits TypeSize = Context.getTypeSizeInChars(T: BTy);
1930
1931 if (!llvm::isPowerOf2_64(Value: TypeSize.getQuantity())) {
1932 assert(
1933 !Context.getTargetInfo().getTriple().isWindowsMSVCEnvironment() &&
1934 "Non PowerOf2 size in MSVC mode");
1935 // Base types with sizes that aren't a power of two don't work
1936 // with the layout rules for MS structs. This isn't an issue in
1937 // MSVC itself since there are no such base data types there.
1938 // On e.g. x86_32 mingw and linux, long double is 12 bytes though.
1939 // Any structs involving that data type obviously can't be ABI
1940 // compatible with MSVC regardless of how it is laid out.
1941
1942 // Since ms_struct can be mass enabled (via a pragma or via the
1943 // -mms-bitfields command line parameter), this can trigger for
1944 // structs that don't actually need MSVC compatibility, so we
1945 // need to be able to sidestep the ms_struct layout for these types.
1946
1947 // Since the combination of -mms-bitfields together with structs
1948 // like max_align_t (which contains a long double) for mingw is
1949 // quite common (and GCC handles it silently), just handle it
1950 // silently there. For other targets that have ms_struct enabled
1951 // (most probably via a pragma or attribute), trigger a diagnostic
1952 // that defaults to an error.
1953 if (!Context.getTargetInfo().getTriple().isOSCygMing())
1954 Diag(Loc: D->getLocation(), DiagID: diag::warn_npot_ms_struct);
1955 }
1956 if (TypeSize > FieldAlign &&
1957 llvm::isPowerOf2_64(Value: TypeSize.getQuantity()))
1958 FieldAlign = TypeSize;
1959 }
1960 }
1961 }
1962
1963 bool FieldPacked =
1964 (Packed &&
1965 (!FieldClass || FieldClass->isPOD() ||
1966 FieldClass->hasAttr<PackedAttr>() ||
1967 Context.getLangOpts().isCompatibleWith(Version: LangOptions::ClangABI::Ver15) ||
1968 Target.isPS() || Target.isOSDarwin() || Target.isOSAIX())) ||
1969 D->hasAttr<PackedAttr>();
1970
1971 // When used as part of a typedef, or together with a 'packed' attribute, the
1972 // 'aligned' attribute can be used to decrease alignment. In that case, it
1973 // overrides any computed alignment we have, and there is no need to upgrade
1974 // the alignment.
1975 auto alignedAttrCanDecreaseAIXAlignment = [AlignRequirement, FieldPacked] {
1976 // Enum alignment sources can be safely ignored here, because this only
1977 // helps decide whether we need the AIX alignment upgrade, which only
1978 // applies to floating-point types.
1979 return AlignRequirement == AlignRequirementKind::RequiredByTypedef ||
1980 (AlignRequirement == AlignRequirementKind::RequiredByRecord &&
1981 FieldPacked);
1982 };
1983
1984 // The AIX `power` alignment rules apply the natural alignment of the
1985 // "first member" if it is of a floating-point data type (or is an aggregate
1986 // whose recursively "first" member or element is such a type). The alignment
1987 // associated with these types for subsequent members use an alignment value
1988 // where the floating-point data type is considered to have 4-byte alignment.
1989 //
1990 // For the purposes of the foregoing: vtable pointers, non-empty base classes,
1991 // and zero-width bit-fields count as prior members; members of empty class
1992 // types marked `no_unique_address` are not considered to be prior members.
1993 CharUnits PreferredAlign = FieldAlign;
1994 if (DefaultsToAIXPowerAlignment && !alignedAttrCanDecreaseAIXAlignment() &&
1995 (FoundFirstNonOverlappingEmptyFieldForAIX || IsNaturalAlign)) {
1996 auto performBuiltinTypeAlignmentUpgrade = [&](const BuiltinType *BTy) {
1997 if (BTy->getKind() == BuiltinType::Double ||
1998 BTy->getKind() == BuiltinType::LongDouble) {
1999 assert(PreferredAlign == CharUnits::fromQuantity(4) &&
2000 "No need to upgrade the alignment value.");
2001 PreferredAlign = CharUnits::fromQuantity(Quantity: 8);
2002 }
2003 };
2004
2005 const Type *BaseTy = D->getType()->getBaseElementTypeUnsafe();
2006 if (const ComplexType *CTy = BaseTy->getAs<ComplexType>()) {
2007 performBuiltinTypeAlignmentUpgrade(
2008 CTy->getElementType()->castAs<BuiltinType>());
2009 } else if (const BuiltinType *BTy = BaseTy->getAs<BuiltinType>()) {
2010 performBuiltinTypeAlignmentUpgrade(BTy);
2011 } else if (const RecordType *RT = BaseTy->getAsCanonical<RecordType>()) {
2012 const RecordDecl *RD = RT->getDecl();
2013 const ASTRecordLayout &FieldRecord = Context.getASTRecordLayout(D: RD);
2014 PreferredAlign = FieldRecord.getPreferredAlignment();
2015 }
2016 }
2017
2018 // The align if the field is not packed. This is to check if the attribute
2019 // was unnecessary (-Wpacked).
2020 CharUnits UnpackedFieldAlign = FieldAlign;
2021 CharUnits PackedFieldAlign = CharUnits::One();
2022 CharUnits UnpackedFieldOffset = FieldOffset;
2023 CharUnits OriginalFieldAlign = UnpackedFieldAlign;
2024
2025 CharUnits MaxAlignmentInChars =
2026 Context.toCharUnitsFromBits(BitSize: D->getMaxAlignment());
2027 PackedFieldAlign = std::max(a: PackedFieldAlign, b: MaxAlignmentInChars);
2028 PreferredAlign = std::max(a: PreferredAlign, b: MaxAlignmentInChars);
2029 UnpackedFieldAlign = std::max(a: UnpackedFieldAlign, b: MaxAlignmentInChars);
2030
2031 // The maximum field alignment overrides the aligned attribute.
2032 if (!MaxFieldAlignment.isZero()) {
2033 PackedFieldAlign = std::min(a: PackedFieldAlign, b: MaxFieldAlignment);
2034 PreferredAlign = std::min(a: PreferredAlign, b: MaxFieldAlignment);
2035 UnpackedFieldAlign = std::min(a: UnpackedFieldAlign, b: MaxFieldAlignment);
2036 }
2037
2038
2039 if (!FieldPacked)
2040 FieldAlign = UnpackedFieldAlign;
2041 if (DefaultsToAIXPowerAlignment)
2042 UnpackedFieldAlign = PreferredAlign;
2043 if (FieldPacked) {
2044 PreferredAlign = PackedFieldAlign;
2045 FieldAlign = PackedFieldAlign;
2046 }
2047
2048 CharUnits AlignTo =
2049 !DefaultsToAIXPowerAlignment ? FieldAlign : PreferredAlign;
2050 // Round up the current record size to the field's alignment boundary.
2051 FieldOffset = FieldOffset.alignTo(Align: AlignTo);
2052 UnpackedFieldOffset = UnpackedFieldOffset.alignTo(Align: UnpackedFieldAlign);
2053
2054 if (UseExternalLayout) {
2055 FieldOffset = Context.toCharUnitsFromBits(
2056 BitSize: updateExternalFieldOffset(Field: D, ComputedOffset: Context.toBits(CharSize: FieldOffset)));
2057
2058 if (!IsUnion && EmptySubobjects) {
2059 // Record the fact that we're placing a field at this offset.
2060 bool Allowed = EmptySubobjects->CanPlaceFieldAtOffset(FD: D, Offset: FieldOffset);
2061 (void)Allowed;
2062 assert(Allowed && "Externally-placed field cannot be placed here");
2063 }
2064 } else {
2065 if (!IsUnion && EmptySubobjects) {
2066 // Check if we can place the field at this offset.
2067 while (!EmptySubobjects->CanPlaceFieldAtOffset(FD: D, Offset: FieldOffset)) {
2068 // We couldn't place the field at the offset. Try again at a new offset.
2069 // We try offset 0 (for an empty field) and then dsize(C) onwards.
2070 if (FieldOffset == CharUnits::Zero() &&
2071 getDataSize() != CharUnits::Zero())
2072 FieldOffset = getDataSize().alignTo(Align: AlignTo);
2073 else
2074 FieldOffset += AlignTo;
2075 }
2076 }
2077 }
2078
2079 // Place this field at the current location.
2080 FieldOffsets.push_back(Elt: Context.toBits(CharSize: FieldOffset));
2081
2082 if (!UseExternalLayout)
2083 CheckFieldPadding(Offset: Context.toBits(CharSize: FieldOffset), UnpaddedOffset: UnpaddedFieldOffset,
2084 UnpackedOffset: Context.toBits(CharSize: UnpackedFieldOffset),
2085 UnpackedAlign: Context.toBits(CharSize: UnpackedFieldAlign), isPacked: FieldPacked, D);
2086
2087 if (InsertExtraPadding) {
2088 CharUnits ASanAlignment = CharUnits::fromQuantity(Quantity: 8);
2089 CharUnits ExtraSizeForAsan = ASanAlignment;
2090 if (!FieldSize.isMultipleOf(N: ASanAlignment))
2091 ExtraSizeForAsan += ASanAlignment - (FieldSize % ASanAlignment);
2092 EffectiveFieldSize = FieldSize = FieldSize + ExtraSizeForAsan;
2093 }
2094
2095 // Reserve space for this field.
2096 if (!IsOverlappingEmptyField) {
2097 uint64_t EffectiveFieldSizeInBits = Context.toBits(CharSize: EffectiveFieldSize);
2098 if (IsUnion)
2099 setDataSize(std::max(a: getDataSizeInBits(), b: EffectiveFieldSizeInBits));
2100 else
2101 setDataSize(FieldOffset + EffectiveFieldSize);
2102
2103 PaddedFieldSize = std::max(a: PaddedFieldSize, b: FieldOffset + FieldSize);
2104 setSize(std::max(a: getSizeInBits(), b: getDataSizeInBits()));
2105 } else {
2106 setSize(std::max(a: getSizeInBits(),
2107 b: (uint64_t)Context.toBits(CharSize: FieldOffset + FieldSize)));
2108 }
2109
2110 // Remember max struct/class ABI-specified alignment.
2111 UnadjustedAlignment = std::max(a: UnadjustedAlignment, b: FieldAlign);
2112 UpdateAlignment(NewAlignment: FieldAlign, UnpackedNewAlignment: UnpackedFieldAlign, PreferredAlignment: PreferredAlign);
2113
2114 // For checking the alignment of inner fields against
2115 // the alignment of its parent record.
2116 if (const RecordDecl *RD = D->getParent()) {
2117 // Check if packed attribute or pragma pack is present.
2118 if (RD->hasAttr<PackedAttr>() || !MaxFieldAlignment.isZero())
2119 if (FieldAlign < OriginalFieldAlign)
2120 if (D->getType()->isRecordType()) {
2121 // If the offset is not a multiple of the alignment of
2122 // the type, raise the warning.
2123 // TODO: Takes no account the alignment of the outer struct
2124 if (!FieldOffset.isMultipleOf(N: OriginalFieldAlign))
2125 Diag(Loc: D->getLocation(), DiagID: diag::warn_unaligned_access)
2126 << Context.getCanonicalTagType(TD: RD) << D->getName()
2127 << D->getType();
2128 }
2129 }
2130
2131 if (Packed && !FieldPacked && PackedFieldAlign < FieldAlign)
2132 Diag(Loc: D->getLocation(), DiagID: diag::warn_unpacked_field) << D;
2133}
2134
2135void ItaniumRecordLayoutBuilder::FinishLayout(const NamedDecl *D) {
2136 // In C++, records cannot be of size 0.
2137 if (Context.getLangOpts().CPlusPlus && getSizeInBits() == 0) {
2138 if (const CXXRecordDecl *RD = dyn_cast<CXXRecordDecl>(Val: D)) {
2139 // Compatibility with gcc requires a class (pod or non-pod)
2140 // which is not empty but of size 0; such as having fields of
2141 // array of zero-length, remains of Size 0
2142 if (RD->isEmpty())
2143 setSize(CharUnits::One());
2144 }
2145 else
2146 setSize(CharUnits::One());
2147 }
2148
2149 // If we have any remaining field tail padding, include that in the overall
2150 // size.
2151 setSize(std::max(a: getSizeInBits(), b: (uint64_t)Context.toBits(CharSize: PaddedFieldSize)));
2152
2153 // Finally, round the size of the record up to the alignment of the
2154 // record itself.
2155 uint64_t UnpaddedSize = getSizeInBits() - UnfilledBitsInLastUnit;
2156 uint64_t UnpackedSizeInBits =
2157 llvm::alignTo(Value: getSizeInBits(), Align: Context.toBits(CharSize: UnpackedAlignment));
2158
2159 uint64_t RoundedSize = llvm::alignTo(
2160 Value: getSizeInBits(),
2161 Align: Context.toBits(CharSize: !Context.getTargetInfo().defaultsToAIXPowerAlignment()
2162 ? Alignment
2163 : PreferredAlignment));
2164
2165 if (UseExternalLayout) {
2166 // If we're inferring alignment, and the external size is smaller than
2167 // our size after we've rounded up to alignment, conservatively set the
2168 // alignment to 1.
2169 if (InferAlignment && External.Size < RoundedSize) {
2170 Alignment = CharUnits::One();
2171 PreferredAlignment = CharUnits::One();
2172 InferAlignment = false;
2173 }
2174 setSize(External.Size);
2175 return;
2176 }
2177
2178 // Set the size to the final size.
2179 setSize(RoundedSize);
2180
2181 unsigned CharBitNum = Context.getTargetInfo().getCharWidth();
2182 if (const RecordDecl *RD = dyn_cast<RecordDecl>(Val: D)) {
2183 // Warn if padding was introduced to the struct/class/union.
2184 if (getSizeInBits() > UnpaddedSize) {
2185 unsigned PadSize = getSizeInBits() - UnpaddedSize;
2186 bool InBits = true;
2187 if (PadSize % CharBitNum == 0) {
2188 PadSize = PadSize / CharBitNum;
2189 InBits = false;
2190 }
2191 Diag(Loc: RD->getLocation(), DiagID: diag::warn_padded_struct_size)
2192 << Context.getCanonicalTagType(TD: RD) << PadSize
2193 << (InBits ? 1 : 0); // (byte|bit)
2194 }
2195
2196 const auto *CXXRD = dyn_cast<CXXRecordDecl>(Val: RD);
2197
2198 // Warn if we packed it unnecessarily, when the unpacked alignment is not
2199 // greater than the one after packing, the size in bits doesn't change and
2200 // the offset of each field is identical.
2201 // Unless the type is non-POD (for Clang ABI > 15), where the packed
2202 // attribute on such a type does allow the type to be packed into other
2203 // structures that use the packed attribute.
2204 if (Packed && UnpackedAlignment <= Alignment &&
2205 UnpackedSizeInBits == getSizeInBits() && !HasPackedField &&
2206 (!CXXRD || CXXRD->isPOD() ||
2207 Context.getLangOpts().isCompatibleWith(Version: LangOptions::ClangABI::Ver15)))
2208 Diag(Loc: D->getLocation(), DiagID: diag::warn_unnecessary_packed)
2209 << Context.getCanonicalTagType(TD: RD);
2210 }
2211}
2212
2213void ItaniumRecordLayoutBuilder::UpdateAlignment(
2214 CharUnits NewAlignment, CharUnits UnpackedNewAlignment,
2215 CharUnits PreferredNewAlignment) {
2216 // The alignment is not modified when using 'mac68k' alignment or when
2217 // we have an externally-supplied layout that also provides overall alignment.
2218 if (IsMac68kAlign || (UseExternalLayout && !InferAlignment))
2219 return;
2220
2221 if (NewAlignment > Alignment) {
2222 assert(llvm::isPowerOf2_64(NewAlignment.getQuantity()) &&
2223 "Alignment not a power of 2");
2224 Alignment = NewAlignment;
2225 }
2226
2227 if (UnpackedNewAlignment > UnpackedAlignment) {
2228 assert(llvm::isPowerOf2_64(UnpackedNewAlignment.getQuantity()) &&
2229 "Alignment not a power of 2");
2230 UnpackedAlignment = UnpackedNewAlignment;
2231 }
2232
2233 if (PreferredNewAlignment > PreferredAlignment) {
2234 assert(llvm::isPowerOf2_64(PreferredNewAlignment.getQuantity()) &&
2235 "Alignment not a power of 2");
2236 PreferredAlignment = PreferredNewAlignment;
2237 }
2238}
2239
2240uint64_t
2241ItaniumRecordLayoutBuilder::updateExternalFieldOffset(const FieldDecl *Field,
2242 uint64_t ComputedOffset) {
2243 uint64_t ExternalFieldOffset = External.getExternalFieldOffset(FD: Field);
2244
2245 if (InferAlignment && ExternalFieldOffset < ComputedOffset) {
2246 // The externally-supplied field offset is before the field offset we
2247 // computed. Assume that the structure is packed.
2248 Alignment = CharUnits::One();
2249 PreferredAlignment = CharUnits::One();
2250 InferAlignment = false;
2251 }
2252
2253 // Use the externally-supplied field offset.
2254 return ExternalFieldOffset;
2255}
2256
2257/// Get diagnostic %select index for tag kind for
2258/// field padding diagnostic message.
2259/// WARNING: Indexes apply to particular diagnostics only!
2260///
2261/// \returns diagnostic %select index.
2262static unsigned getPaddingDiagFromTagKind(TagTypeKind Tag) {
2263 switch (Tag) {
2264 case TagTypeKind::Struct:
2265 return 0;
2266 case TagTypeKind::Interface:
2267 return 1;
2268 case TagTypeKind::Class:
2269 return 2;
2270 default: llvm_unreachable("Invalid tag kind for field padding diagnostic!");
2271 }
2272}
2273
2274static void CheckFieldPadding(const ASTContext &Context, bool IsUnion,
2275 uint64_t Offset, uint64_t UnpaddedOffset,
2276 const FieldDecl *D) {
2277 // We let objc ivars without warning, objc interfaces generally are not used
2278 // for padding tricks.
2279 if (isa<ObjCIvarDecl>(Val: D))
2280 return;
2281
2282 // Don't warn about structs created without a SourceLocation. This can
2283 // be done by clients of the AST, such as codegen.
2284 if (D->getLocation().isInvalid())
2285 return;
2286
2287 unsigned CharBitNum = Context.getTargetInfo().getCharWidth();
2288
2289 // Warn if padding was introduced to the struct/class.
2290 if (!IsUnion && Offset > UnpaddedOffset) {
2291 unsigned PadSize = Offset - UnpaddedOffset;
2292 bool InBits = true;
2293 if (PadSize % CharBitNum == 0) {
2294 PadSize = PadSize / CharBitNum;
2295 InBits = false;
2296 }
2297 if (D->getIdentifier()) {
2298 auto Diagnostic = D->isBitField() ? diag::warn_padded_struct_bitfield
2299 : diag::warn_padded_struct_field;
2300 Context.getDiagnostics().Report(Loc: D->getLocation(),
2301 DiagID: Diagnostic)
2302 << getPaddingDiagFromTagKind(Tag: D->getParent()->getTagKind())
2303 << Context.getCanonicalTagType(TD: D->getParent()) << PadSize
2304 << (InBits ? 1 : 0) // (byte|bit)
2305 << D->getIdentifier();
2306 } else {
2307 auto Diagnostic = D->isBitField() ? diag::warn_padded_struct_anon_bitfield
2308 : diag::warn_padded_struct_anon_field;
2309 Context.getDiagnostics().Report(Loc: D->getLocation(),
2310 DiagID: Diagnostic)
2311 << getPaddingDiagFromTagKind(Tag: D->getParent()->getTagKind())
2312 << Context.getCanonicalTagType(TD: D->getParent()) << PadSize
2313 << (InBits ? 1 : 0); // (byte|bit)
2314 }
2315 }
2316}
2317
2318void ItaniumRecordLayoutBuilder::CheckFieldPadding(
2319 uint64_t Offset, uint64_t UnpaddedOffset, uint64_t UnpackedOffset,
2320 unsigned UnpackedAlign, bool isPacked, const FieldDecl *D) {
2321 ::CheckFieldPadding(Context, IsUnion, Offset, UnpaddedOffset, D);
2322 if (isPacked && Offset != UnpackedOffset) {
2323 HasPackedField = true;
2324 }
2325}
2326
2327static const CXXMethodDecl *computeKeyFunction(ASTContext &Context,
2328 const CXXRecordDecl *RD) {
2329 // If a class isn't polymorphic it doesn't have a key function.
2330 if (!RD->isPolymorphic())
2331 return nullptr;
2332
2333 // A class that is not externally visible doesn't have a key function. (Or
2334 // at least, there's no point to assigning a key function to such a class;
2335 // this doesn't affect the ABI.)
2336 if (!RD->isExternallyVisible())
2337 return nullptr;
2338
2339 // Template instantiations don't have key functions per Itanium C++ ABI 5.2.6.
2340 // Same behavior as GCC.
2341 TemplateSpecializationKind TSK = RD->getTemplateSpecializationKind();
2342 if (TSK == TSK_ImplicitInstantiation ||
2343 TSK == TSK_ExplicitInstantiationDeclaration ||
2344 TSK == TSK_ExplicitInstantiationDefinition)
2345 return nullptr;
2346
2347 bool allowInlineFunctions =
2348 Context.getTargetInfo().getCXXABI().canKeyFunctionBeInline();
2349
2350 for (const CXXMethodDecl *MD : RD->methods()) {
2351 if (!MD->isVirtual())
2352 continue;
2353
2354 if (MD->isPureVirtual())
2355 continue;
2356
2357 // Ignore implicit member functions, they are always marked as inline, but
2358 // they don't have a body until they're defined.
2359 if (MD->isImplicit())
2360 continue;
2361
2362 if (MD->isInlineSpecified() || MD->isConstexpr())
2363 continue;
2364
2365 if (MD->hasInlineBody())
2366 continue;
2367
2368 // Ignore inline deleted or defaulted functions.
2369 if (!MD->isUserProvided())
2370 continue;
2371
2372 // In certain ABIs, ignore functions with out-of-line inline definitions.
2373 if (!allowInlineFunctions) {
2374 const FunctionDecl *Def;
2375 if (MD->hasBody(Definition&: Def) && Def->isInlineSpecified())
2376 continue;
2377 }
2378
2379 if (Context.getLangOpts().CUDA) {
2380 // While compiler may see key method in this TU, during CUDA
2381 // compilation we should ignore methods that are not accessible
2382 // on this side of compilation.
2383 if (Context.getLangOpts().CUDAIsDevice) {
2384 // In device mode ignore methods without __device__ attribute.
2385 if (!MD->hasAttr<CUDADeviceAttr>())
2386 continue;
2387 } else {
2388 // In host mode ignore __device__-only methods.
2389 if (!MD->hasAttr<CUDAHostAttr>() && MD->hasAttr<CUDADeviceAttr>())
2390 continue;
2391 }
2392 }
2393
2394 // If the key function is dllimport but the class isn't, then the class has
2395 // no key function. The DLL that exports the key function won't export the
2396 // vtable in this case.
2397 if (MD->hasAttr<DLLImportAttr>() && !RD->hasAttr<DLLImportAttr>() &&
2398 !Context.getTargetInfo().hasPS4DLLImportExport())
2399 return nullptr;
2400
2401 // We found it.
2402 return MD;
2403 }
2404
2405 return nullptr;
2406}
2407
2408DiagnosticBuilder ItaniumRecordLayoutBuilder::Diag(SourceLocation Loc,
2409 unsigned DiagID) {
2410 return Context.getDiagnostics().Report(Loc, DiagID);
2411}
2412
2413/// Does the target C++ ABI require us to skip over the tail-padding
2414/// of the given class (considering it as a base class) when allocating
2415/// objects?
2416static bool mustSkipTailPadding(TargetCXXABI ABI, const CXXRecordDecl *RD) {
2417 switch (ABI.getTailPaddingUseRules()) {
2418 case TargetCXXABI::AlwaysUseTailPadding:
2419 return false;
2420
2421 case TargetCXXABI::UseTailPaddingUnlessPOD03:
2422 // FIXME: To the extent that this is meant to cover the Itanium ABI
2423 // rules, we should implement the restrictions about over-sized
2424 // bitfields:
2425 //
2426 // http://itanium-cxx-abi.github.io/cxx-abi/abi.html#POD :
2427 // In general, a type is considered a POD for the purposes of
2428 // layout if it is a POD type (in the sense of ISO C++
2429 // [basic.types]). However, a POD-struct or POD-union (in the
2430 // sense of ISO C++ [class]) with a bitfield member whose
2431 // declared width is wider than the declared type of the
2432 // bitfield is not a POD for the purpose of layout. Similarly,
2433 // an array type is not a POD for the purpose of layout if the
2434 // element type of the array is not a POD for the purpose of
2435 // layout.
2436 //
2437 // Where references to the ISO C++ are made in this paragraph,
2438 // the Technical Corrigendum 1 version of the standard is
2439 // intended.
2440 return RD->isPOD();
2441
2442 case TargetCXXABI::UseTailPaddingUnlessPOD11:
2443 // This is equivalent to RD->getTypeForDecl().isCXX11PODType(),
2444 // but with a lot of abstraction penalty stripped off. This does
2445 // assume that these properties are set correctly even in C++98
2446 // mode; fortunately, that is true because we want to assign
2447 // consistently semantics to the type-traits intrinsics (or at
2448 // least as many of them as possible).
2449 return RD->isTrivial() && RD->isCXX11StandardLayout();
2450 }
2451
2452 llvm_unreachable("bad tail-padding use kind");
2453}
2454
2455// This section contains an implementation of struct layout that is, up to the
2456// included tests, compatible with cl.exe (2013). The layout produced is
2457// significantly different than those produced by the Itanium ABI. Here we note
2458// the most important differences.
2459//
2460// * The alignment of bitfields in unions is ignored when computing the
2461// alignment of the union.
2462// * The existence of zero-width bitfield that occurs after anything other than
2463// a non-zero length bitfield is ignored.
2464// * There is no explicit primary base for the purposes of layout. All bases
2465// with vfptrs are laid out first, followed by all bases without vfptrs.
2466// * The Itanium equivalent vtable pointers are split into a vfptr (virtual
2467// function pointer) and a vbptr (virtual base pointer). They can each be
2468// shared with a, non-virtual bases. These bases need not be the same. vfptrs
2469// always occur at offset 0. vbptrs can occur at an arbitrary offset and are
2470// placed after the lexicographically last non-virtual base. This placement
2471// is always before fields but can be in the middle of the non-virtual bases
2472// due to the two-pass layout scheme for non-virtual-bases.
2473// * Virtual bases sometimes require a 'vtordisp' field that is laid out before
2474// the virtual base and is used in conjunction with virtual overrides during
2475// construction and destruction. This is always a 4 byte value and is used as
2476// an alternative to constructor vtables.
2477// * vtordisps are allocated in a block of memory with size and alignment equal
2478// to the alignment of the completed structure (before applying __declspec(
2479// align())). The vtordisp always occur at the end of the allocation block,
2480// immediately prior to the virtual base.
2481// * vfptrs are injected after all bases and fields have been laid out. In
2482// order to guarantee proper alignment of all fields, the vfptr injection
2483// pushes all bases and fields back by the alignment imposed by those bases
2484// and fields. This can potentially add a significant amount of padding.
2485// vfptrs are always injected at offset 0.
2486// * vbptrs are injected after all bases and fields have been laid out. In
2487// order to guarantee proper alignment of all fields, the vfptr injection
2488// pushes all bases and fields back by the alignment imposed by those bases
2489// and fields. This can potentially add a significant amount of padding.
2490// vbptrs are injected immediately after the last non-virtual base as
2491// lexicographically ordered in the code. If this site isn't pointer aligned
2492// the vbptr is placed at the next properly aligned location. Enough padding
2493// is added to guarantee a fit.
2494// * The last zero sized non-virtual base can be placed at the end of the
2495// struct (potentially aliasing another object), or may alias with the first
2496// field, even if they are of the same type.
2497// * The last zero size virtual base may be placed at the end of the struct
2498// potentially aliasing another object.
2499// * The ABI attempts to avoid aliasing of zero sized bases by adding padding
2500// between bases or vbases with specific properties. The criteria for
2501// additional padding between two bases is that the first base is zero sized
2502// or ends with a zero sized subobject and the second base is zero sized or
2503// trails with a zero sized base or field (sharing of vfptrs can reorder the
2504// layout of the so the leading base is not always the first one declared).
2505// This rule does take into account fields that are not records, so padding
2506// will occur even if the last field is, e.g. an int. The padding added for
2507// bases is 1 byte. The padding added between vbases depends on the alignment
2508// of the object but is at least 4 bytes (in both 32 and 64 bit modes).
2509// * There is no concept of non-virtual alignment, non-virtual alignment and
2510// alignment are always identical.
2511// * There is a distinction between alignment and required alignment.
2512// __declspec(align) changes the required alignment of a struct. This
2513// alignment is _always_ obeyed, even in the presence of #pragma pack. A
2514// record inherits required alignment from all of its fields and bases.
2515// * __declspec(align) on bitfields has the effect of changing the bitfield's
2516// alignment instead of its required alignment. This is the only known way
2517// to make the alignment of a struct bigger than 8. Interestingly enough
2518// this alignment is also immune to the effects of #pragma pack and can be
2519// used to create structures with large alignment under #pragma pack.
2520// However, because it does not impact required alignment, such a structure,
2521// when used as a field or base, will not be aligned if #pragma pack is
2522// still active at the time of use.
2523//
2524// Known incompatibilities:
2525// * all: #pragma pack between fields in a record
2526// * 2010 and back: If the last field in a record is a bitfield, every object
2527// laid out after the record will have extra padding inserted before it. The
2528// extra padding will have size equal to the size of the storage class of the
2529// bitfield. 0 sized bitfields don't exhibit this behavior and the extra
2530// padding can be avoided by adding a 0 sized bitfield after the non-zero-
2531// sized bitfield.
2532// * 2012 and back: In 64-bit mode, if the alignment of a record is 16 or
2533// greater due to __declspec(align()) then a second layout phase occurs after
2534// The locations of the vf and vb pointers are known. This layout phase
2535// suffers from the "last field is a bitfield" bug in 2010 and results in
2536// _every_ field getting padding put in front of it, potentially including the
2537// vfptr, leaving the vfprt at a non-zero location which results in a fault if
2538// anything tries to read the vftbl. The second layout phase also treats
2539// bitfields as separate entities and gives them each storage rather than
2540// packing them. Additionally, because this phase appears to perform a
2541// (an unstable) sort on the members before laying them out and because merged
2542// bitfields have the same address, the bitfields end up in whatever order
2543// the sort left them in, a behavior we could never hope to replicate.
2544
2545namespace {
2546struct MicrosoftRecordLayoutBuilder {
2547 struct ElementInfo {
2548 CharUnits Size;
2549 CharUnits Alignment;
2550 /// The alignment to use when updating the enclosing record's alignment.
2551 /// This differs from Alignment only on targets that reuse over-aligned tail
2552 /// padding, where over-alignment applied directly to a field must not raise
2553 /// the record's alignment (only its required alignment).
2554 CharUnits NonRequiredAlignment;
2555 };
2556 typedef llvm::DenseMap<const CXXRecordDecl *, CharUnits> BaseOffsetsMapTy;
2557 MicrosoftRecordLayoutBuilder(const ASTContext &Context,
2558 EmptySubobjectMap *EmptySubobjects)
2559 : Context(Context), EmptySubobjects(EmptySubobjects),
2560 RemainingBitsInField(0) {}
2561
2562private:
2563 MicrosoftRecordLayoutBuilder(const MicrosoftRecordLayoutBuilder &) = delete;
2564 void operator=(const MicrosoftRecordLayoutBuilder &) = delete;
2565public:
2566 void layout(const RecordDecl *RD);
2567 void cxxLayout(const CXXRecordDecl *RD);
2568 /// Initializes size and alignment and honors some flags.
2569 void initializeLayout(const RecordDecl *RD);
2570 /// Initialized C++ layout, compute alignment and virtual alignment and
2571 /// existence of vfptrs and vbptrs. Alignment is needed before the vfptr is
2572 /// laid out.
2573 void initializeCXXLayout(const CXXRecordDecl *RD);
2574 void layoutNonVirtualBases(const CXXRecordDecl *RD);
2575 void layoutNonVirtualBase(const CXXRecordDecl *RD,
2576 const CXXRecordDecl *BaseDecl,
2577 const ASTRecordLayout &BaseLayout,
2578 const ASTRecordLayout *&PreviousBaseLayout);
2579 void injectVFPtr(const CXXRecordDecl *RD);
2580 void injectVBPtr(const CXXRecordDecl *RD);
2581 /// Lays out the fields of the record. Also rounds size up to
2582 /// alignment.
2583 void layoutFields(const RecordDecl *RD);
2584 void layoutField(const FieldDecl *FD);
2585 void layoutBitField(const FieldDecl *FD);
2586 /// Lays out a single zero-width bit-field in the record and handles
2587 /// special cases associated with zero-width bit-fields.
2588 void layoutZeroWidthBitField(const FieldDecl *FD);
2589 void layoutVirtualBases(const CXXRecordDecl *RD);
2590 void finalizeLayout(const RecordDecl *RD);
2591 /// Gets the size and alignment of a base taking pragma pack and
2592 /// __declspec(align) into account.
2593 ElementInfo getAdjustedElementInfo(const ASTRecordLayout &Layout);
2594 /// Gets the size and alignment of a field taking pragma pack and
2595 /// __declspec(align) into account. It also updates RequiredAlignment as a
2596 /// side effect because it is most convenient to do so here.
2597 ElementInfo getAdjustedElementInfo(const FieldDecl *FD);
2598 /// Returns true if this target reuses the tail padding of an over-aligned
2599 /// base class for a subsequent base. MSVC does this on Arm64, but not on
2600 /// Arm64EC (which follows the x64 layout rules) or x64.
2601 bool reuseOveralignedBaseTailPadding() const {
2602 const llvm::Triple &Triple = Context.getTargetInfo().getTriple();
2603 return Triple.isAArch64() && !Triple.isWindowsArm64EC();
2604 }
2605 /// Places a field at an offset in CharUnits.
2606 void placeFieldAtOffset(CharUnits FieldOffset) {
2607 FieldOffsets.push_back(Elt: Context.toBits(CharSize: FieldOffset));
2608 }
2609 /// Places a bitfield at a bit offset.
2610 void placeFieldAtBitOffset(uint64_t FieldOffset) {
2611 FieldOffsets.push_back(Elt: FieldOffset);
2612 }
2613 /// Compute the set of virtual bases for which vtordisps are required.
2614 void computeVtorDispSet(
2615 llvm::SmallPtrSetImpl<const CXXRecordDecl *> &HasVtorDispSet,
2616 const CXXRecordDecl *RD) const;
2617 const ASTContext &Context;
2618 EmptySubobjectMap *EmptySubobjects;
2619
2620 /// The size of the record being laid out.
2621 CharUnits Size;
2622 /// The non-virtual size of the record layout.
2623 CharUnits NonVirtualSize;
2624 /// The data size of the record layout.
2625 CharUnits DataSize;
2626 /// The current alignment of the record layout.
2627 CharUnits Alignment;
2628 /// The maximum allowed field alignment. This is set by #pragma pack.
2629 CharUnits MaxFieldAlignment;
2630 /// The alignment that this record must obey. This is imposed by
2631 /// __declspec(align()) on the record itself or one of its fields or bases.
2632 CharUnits RequiredAlignment;
2633 /// The alignment of the record ignoring any over-alignment imposed via
2634 /// __declspec(align()) / alignas on the record or its bases.
2635 CharUnits NonRequiredAlignment;
2636 /// The size of the allocation of the currently active bitfield.
2637 /// This value isn't meaningful unless LastFieldIsNonZeroWidthBitfield
2638 /// is true.
2639 CharUnits CurrentBitfieldSize;
2640 /// Offset to the virtual base table pointer (if one exists).
2641 CharUnits VBPtrOffset;
2642 /// Minimum record size possible.
2643 CharUnits MinEmptyStructSize;
2644 /// The size and alignment info of a pointer.
2645 ElementInfo PointerInfo;
2646 /// The primary base class (if one exists).
2647 const CXXRecordDecl *PrimaryBase;
2648 /// The class we share our vb-pointer with.
2649 const CXXRecordDecl *SharedVBPtrBase;
2650 /// The collection of field offsets.
2651 SmallVector<uint64_t, 16> FieldOffsets;
2652 /// Base classes and their offsets in the record.
2653 BaseOffsetsMapTy Bases;
2654 /// virtual base classes and their offsets in the record.
2655 ASTRecordLayout::VBaseOffsetsMapTy VBases;
2656 /// The number of remaining bits in our last bitfield allocation.
2657 unsigned RemainingBitsInField;
2658 bool IsUnion : 1;
2659 /// True if the last field laid out was a bitfield and was not 0
2660 /// width.
2661 bool LastFieldIsNonZeroWidthBitfield : 1;
2662 /// True if the class has its own vftable pointer.
2663 bool HasOwnVFPtr : 1;
2664 /// True if the class has a vbtable pointer.
2665 bool HasVBPtr : 1;
2666 /// True if the last sub-object within the type is zero sized or the
2667 /// object itself is zero sized. This *does not* count members that are not
2668 /// records. Only used for MS-ABI.
2669 bool EndsWithZeroSizedObject : 1;
2670 /// True if this class is zero sized or first base is zero sized or
2671 /// has this property. Only used for MS-ABI.
2672 bool LeadsWithZeroSizedBase : 1;
2673
2674 /// True if the external AST source provided a layout for this record.
2675 bool UseExternalLayout : 1;
2676
2677 /// The layout provided by the external AST source. Only active if
2678 /// UseExternalLayout is true.
2679 ExternalLayout External;
2680};
2681} // namespace
2682
2683MicrosoftRecordLayoutBuilder::ElementInfo
2684MicrosoftRecordLayoutBuilder::getAdjustedElementInfo(
2685 const ASTRecordLayout &Layout) {
2686 ElementInfo Info;
2687 Info.Alignment = Layout.getAlignment();
2688 // Respect pragma pack.
2689 if (!MaxFieldAlignment.isZero())
2690 Info.Alignment = std::min(a: Info.Alignment, b: MaxFieldAlignment);
2691 // Track zero-sized subobjects here where it's already available.
2692 EndsWithZeroSizedObject = Layout.endsWithZeroSizedObject();
2693 // Respect required alignment, this is necessary because we may have adjusted
2694 // the alignment in the case of pragma pack. Note that the required alignment
2695 // doesn't actually apply to the struct alignment at this point.
2696 // When reusing a base's tail padding, fold in the base's alignment ignoring
2697 // any over-alignment it imposes, so that over-alignment contributes to the
2698 // record's size but not to the alignment its tail padding is rounded to.
2699 CharUnits BaseAlignment = Info.Alignment;
2700 if (reuseOveralignedBaseTailPadding()) {
2701 BaseAlignment = Layout.getNonRequiredNVAlignment();
2702 if (!MaxFieldAlignment.isZero())
2703 BaseAlignment = std::min(a: BaseAlignment, b: MaxFieldAlignment);
2704 }
2705 Alignment = std::max(a: Alignment, b: BaseAlignment);
2706 RequiredAlignment = std::max(a: RequiredAlignment, b: Layout.getRequiredAlignment());
2707 Info.Alignment = std::max(a: Info.Alignment, b: Layout.getRequiredAlignment());
2708 Info.NonRequiredAlignment = Info.Alignment;
2709 Info.Size = Layout.getNonVirtualSize();
2710 return Info;
2711}
2712
2713MicrosoftRecordLayoutBuilder::ElementInfo
2714MicrosoftRecordLayoutBuilder::getAdjustedElementInfo(
2715 const FieldDecl *FD) {
2716 // Get the alignment of the field type's natural alignment, ignore any
2717 // alignment attributes.
2718 auto TInfo =
2719 Context.getTypeInfoInChars(T: FD->getType()->getUnqualifiedDesugaredType());
2720 ElementInfo Info{.Size: TInfo.Width, .Alignment: TInfo.Align, .NonRequiredAlignment: CharUnits::Zero()};
2721 // Respect align attributes on the field.
2722 CharUnits DirectFieldAlignment =
2723 Context.toCharUnitsFromBits(BitSize: FD->getMaxAlignment());
2724 // The portion of the field's required alignment that comes from its type
2725 // rather than from over-alignment applied directly to the field.
2726 CharUnits FieldTypeRequiredAlignment = CharUnits::Zero();
2727 // Respect align attributes on the type.
2728 if (Context.isAlignmentRequired(T: FD->getType()))
2729 FieldTypeRequiredAlignment = Context.getTypeAlignInChars(T: FD->getType());
2730 // Respect attributes applied to subobjects of the field.
2731 if (FD->isBitField())
2732 // For some reason __declspec align impacts alignment rather than required
2733 // alignment when it is applied to bitfields.
2734 Info.Alignment =
2735 std::max(a: Info.Alignment,
2736 b: std::max(a: DirectFieldAlignment, b: FieldTypeRequiredAlignment));
2737 else {
2738 if (const auto *RT = FD->getType()
2739 ->getBaseElementTypeUnsafe()
2740 ->getAsCanonical<RecordType>()) {
2741 auto const &Layout = Context.getASTRecordLayout(D: RT->getDecl());
2742 EndsWithZeroSizedObject = Layout.endsWithZeroSizedObject();
2743 FieldTypeRequiredAlignment =
2744 std::max(a: FieldTypeRequiredAlignment, b: Layout.getRequiredAlignment());
2745 }
2746 // Capture required alignment as a side-effect.
2747 RequiredAlignment =
2748 std::max(a: RequiredAlignment,
2749 b: std::max(a: DirectFieldAlignment, b: FieldTypeRequiredAlignment));
2750 }
2751 // Respect pragma pack, attribute pack and declspec align
2752 if (!MaxFieldAlignment.isZero())
2753 Info.Alignment = std::min(a: Info.Alignment, b: MaxFieldAlignment);
2754 if (FD->hasAttr<PackedAttr>())
2755 Info.Alignment = CharUnits::One();
2756 // The alignment used to update the record's alignment excludes over-alignment
2757 // applied directly to the field; the alignment used for placement includes
2758 // it. On targets that don't reuse over-aligned tail padding the two are the
2759 // same.
2760 Info.NonRequiredAlignment =
2761 std::max(a: Info.Alignment, b: FieldTypeRequiredAlignment);
2762 Info.Alignment = std::max(a: Info.NonRequiredAlignment, b: DirectFieldAlignment);
2763 // Bitfields are excluded: over-alignment applied directly to a bitfield
2764 // raises the record's alignment (not its required alignment) and is immune to
2765 // #pragma pack, so it must not be stripped here. Stripping it would drop
2766 // that alignment when #pragma pack clamps the bitfield's natural alignment
2767 // below the over-alignment.
2768 if (!reuseOveralignedBaseTailPadding() || FD->isBitField())
2769 Info.NonRequiredAlignment = Info.Alignment;
2770 return Info;
2771}
2772
2773void MicrosoftRecordLayoutBuilder::layout(const RecordDecl *RD) {
2774 // For C record layout, zero-sized records always have size 4.
2775 MinEmptyStructSize = CharUnits::fromQuantity(Quantity: 4);
2776 initializeLayout(RD);
2777 layoutFields(RD);
2778 DataSize = Size = Size.alignTo(Align: Alignment);
2779 RequiredAlignment = std::max(
2780 a: RequiredAlignment, b: Context.toCharUnitsFromBits(BitSize: RD->getMaxAlignment()));
2781 finalizeLayout(RD);
2782}
2783
2784void MicrosoftRecordLayoutBuilder::cxxLayout(const CXXRecordDecl *RD) {
2785 // The C++ standard says that empty structs have size 1.
2786 MinEmptyStructSize = CharUnits::One();
2787 initializeLayout(RD);
2788 initializeCXXLayout(RD);
2789 layoutNonVirtualBases(RD);
2790 layoutFields(RD);
2791 injectVBPtr(RD);
2792 injectVFPtr(RD);
2793 if (HasOwnVFPtr || (HasVBPtr && !SharedVBPtrBase))
2794 Alignment = std::max(a: Alignment, b: PointerInfo.Alignment);
2795 auto RoundingAlignment = Alignment;
2796 if (!MaxFieldAlignment.isZero())
2797 RoundingAlignment = std::min(a: RoundingAlignment, b: MaxFieldAlignment);
2798 if (!UseExternalLayout)
2799 Size = Size.alignTo(Align: RoundingAlignment);
2800 NonVirtualSize = Size;
2801 RequiredAlignment = std::max(
2802 a: RequiredAlignment, b: Context.toCharUnitsFromBits(BitSize: RD->getMaxAlignment()));
2803 layoutVirtualBases(RD);
2804 finalizeLayout(RD);
2805}
2806
2807void MicrosoftRecordLayoutBuilder::initializeLayout(const RecordDecl *RD) {
2808 IsUnion = RD->isUnion();
2809 Size = CharUnits::Zero();
2810 Alignment = CharUnits::One();
2811 NonRequiredAlignment = CharUnits::One();
2812 // In 64-bit mode we always perform an alignment step after laying out vbases.
2813 // In 32-bit mode we do not. The check to see if we need to perform alignment
2814 // checks the RequiredAlignment field and performs alignment if it isn't 0.
2815 RequiredAlignment = Context.getTargetInfo().getTriple().isArch64Bit()
2816 ? CharUnits::One()
2817 : CharUnits::Zero();
2818 // Compute the maximum field alignment.
2819 MaxFieldAlignment = CharUnits::Zero();
2820 // Honor the default struct packing maximum alignment flag.
2821 if (unsigned DefaultMaxFieldAlignment = Context.getLangOpts().PackStruct)
2822 MaxFieldAlignment = CharUnits::fromQuantity(Quantity: DefaultMaxFieldAlignment);
2823 // Honor the packing attribute. The MS-ABI ignores pragma pack if its larger
2824 // than the pointer size.
2825 if (const MaxFieldAlignmentAttr *MFAA = RD->getAttr<MaxFieldAlignmentAttr>()){
2826 unsigned PackedAlignment = MFAA->getAlignment();
2827 if (PackedAlignment <=
2828 Context.getTargetInfo().getPointerWidth(AddrSpace: LangAS::Default))
2829 MaxFieldAlignment = Context.toCharUnitsFromBits(BitSize: PackedAlignment);
2830 }
2831 // Packed attribute forces max field alignment to be 1.
2832 if (RD->hasAttr<PackedAttr>())
2833 MaxFieldAlignment = CharUnits::One();
2834
2835 // Try to respect the external layout if present.
2836 UseExternalLayout = false;
2837 if (ExternalASTSource *Source = Context.getExternalSource())
2838 UseExternalLayout = Source->layoutRecordType(
2839 Record: RD, Size&: External.Size, Alignment&: External.Align, FieldOffsets&: External.FieldOffsets,
2840 BaseOffsets&: External.BaseOffsets, VirtualBaseOffsets&: External.VirtualBaseOffsets);
2841
2842 if (!RD->isMsStruct(C: Context)) {
2843 auto Location = RD->getLocation();
2844 if (Location.isValid())
2845 Context.getDiagnostics().Report(Loc: Location,
2846 DiagID: diag::err_itanium_layout_unimplemented);
2847 }
2848}
2849
2850void
2851MicrosoftRecordLayoutBuilder::initializeCXXLayout(const CXXRecordDecl *RD) {
2852 EndsWithZeroSizedObject = false;
2853 LeadsWithZeroSizedBase = false;
2854 HasOwnVFPtr = false;
2855 HasVBPtr = false;
2856 PrimaryBase = nullptr;
2857 SharedVBPtrBase = nullptr;
2858 // Calculate pointer size and alignment. These are used for vfptr and vbprt
2859 // injection.
2860 PointerInfo.Size = Context.toCharUnitsFromBits(
2861 BitSize: Context.getTargetInfo().getPointerWidth(AddrSpace: LangAS::Default));
2862 PointerInfo.Alignment = Context.toCharUnitsFromBits(
2863 BitSize: Context.getTargetInfo().getPointerAlign(AddrSpace: LangAS::Default));
2864 // Respect pragma pack.
2865 if (!MaxFieldAlignment.isZero())
2866 PointerInfo.Alignment = std::min(a: PointerInfo.Alignment, b: MaxFieldAlignment);
2867}
2868
2869void
2870MicrosoftRecordLayoutBuilder::layoutNonVirtualBases(const CXXRecordDecl *RD) {
2871 // The MS-ABI lays out all bases that contain leading vfptrs before it lays
2872 // out any bases that do not contain vfptrs. We implement this as two passes
2873 // over the bases. This approach guarantees that the primary base is laid out
2874 // first. We use these passes to calculate some additional aggregated
2875 // information about the bases, such as required alignment and the presence of
2876 // zero sized members.
2877 const ASTRecordLayout *PreviousBaseLayout = nullptr;
2878 bool HasPolymorphicBaseClass = false;
2879 // Iterate through the bases and lay out the non-virtual ones.
2880 for (const CXXBaseSpecifier &Base : RD->bases()) {
2881 const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
2882 HasPolymorphicBaseClass |= BaseDecl->isPolymorphic();
2883 const ASTRecordLayout &BaseLayout = Context.getASTRecordLayout(D: BaseDecl);
2884 // Mark and skip virtual bases.
2885 if (Base.isVirtual()) {
2886 HasVBPtr = true;
2887 continue;
2888 }
2889 // Check for a base to share a VBPtr with.
2890 if (!SharedVBPtrBase && BaseLayout.hasVBPtr()) {
2891 SharedVBPtrBase = BaseDecl;
2892 HasVBPtr = true;
2893 }
2894 // Only lay out bases with extendable VFPtrs on the first pass.
2895 if (!BaseLayout.hasExtendableVFPtr())
2896 continue;
2897 // If we don't have a primary base, this one qualifies.
2898 if (!PrimaryBase) {
2899 PrimaryBase = BaseDecl;
2900 LeadsWithZeroSizedBase = BaseLayout.leadsWithZeroSizedBase();
2901 }
2902 // Lay out the base.
2903 layoutNonVirtualBase(RD, BaseDecl, BaseLayout, PreviousBaseLayout);
2904 }
2905 // Figure out if we need a fresh VFPtr for this class.
2906 if (RD->isPolymorphic()) {
2907 if (!HasPolymorphicBaseClass)
2908 // This class introduces polymorphism, so we need a vftable to store the
2909 // RTTI information.
2910 HasOwnVFPtr = true;
2911 else if (!PrimaryBase) {
2912 // We have a polymorphic base class but can't extend its vftable. Add a
2913 // new vfptr if we would use any vftable slots.
2914 for (CXXMethodDecl *M : RD->methods()) {
2915 if (MicrosoftVTableContext::hasVtableSlot(MD: M) &&
2916 M->size_overridden_methods() == 0) {
2917 HasOwnVFPtr = true;
2918 break;
2919 }
2920 }
2921 }
2922 }
2923 // If we don't have a primary base then we have a leading object that could
2924 // itself lead with a zero-sized object, something we track.
2925 bool CheckLeadingLayout = !PrimaryBase;
2926 // Iterate through the bases and lay out the non-virtual ones.
2927 for (const CXXBaseSpecifier &Base : RD->bases()) {
2928 if (Base.isVirtual())
2929 continue;
2930 const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
2931 const ASTRecordLayout &BaseLayout = Context.getASTRecordLayout(D: BaseDecl);
2932 // Only lay out bases without extendable VFPtrs on the second pass.
2933 if (BaseLayout.hasExtendableVFPtr()) {
2934 VBPtrOffset = Bases[BaseDecl] + BaseLayout.getNonVirtualSize();
2935 continue;
2936 }
2937 // If this is the first layout, check to see if it leads with a zero sized
2938 // object. If it does, so do we.
2939 if (CheckLeadingLayout) {
2940 CheckLeadingLayout = false;
2941 LeadsWithZeroSizedBase = BaseLayout.leadsWithZeroSizedBase();
2942 }
2943 // Lay out the base.
2944 layoutNonVirtualBase(RD, BaseDecl, BaseLayout, PreviousBaseLayout);
2945 VBPtrOffset = Bases[BaseDecl] + BaseLayout.getNonVirtualSize();
2946 }
2947 // Set our VBPtroffset if we know it at this point.
2948 if (!HasVBPtr)
2949 VBPtrOffset = CharUnits::fromQuantity(Quantity: -1);
2950 else if (SharedVBPtrBase) {
2951 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: SharedVBPtrBase);
2952 VBPtrOffset = Bases[SharedVBPtrBase] + Layout.getVBPtrOffset();
2953 }
2954}
2955
2956static bool recordUsesEBO(const RecordDecl *RD) {
2957 if (!isa<CXXRecordDecl>(Val: RD))
2958 return false;
2959 if (RD->hasAttr<EmptyBasesAttr>())
2960 return true;
2961 if (auto *LVA = RD->getAttr<LayoutVersionAttr>())
2962 // TODO: Double check with the next version of MSVC.
2963 if (LVA->getVersion() <= LangOptions::MSVC2015)
2964 return false;
2965 // TODO: Some later version of MSVC will change the default behavior of the
2966 // compiler to enable EBO by default. When this happens, we will need an
2967 // additional isCompatibleWithMSVC check.
2968 return false;
2969}
2970
2971void MicrosoftRecordLayoutBuilder::layoutNonVirtualBase(
2972 const CXXRecordDecl *RD, const CXXRecordDecl *BaseDecl,
2973 const ASTRecordLayout &BaseLayout,
2974 const ASTRecordLayout *&PreviousBaseLayout) {
2975 // Insert padding between two bases if the left first one is zero sized or
2976 // contains a zero sized subobject and the right is zero sized or one leads
2977 // with a zero sized base.
2978 bool MDCUsesEBO = recordUsesEBO(RD);
2979 if (PreviousBaseLayout && PreviousBaseLayout->endsWithZeroSizedObject() &&
2980 BaseLayout.leadsWithZeroSizedBase() && !MDCUsesEBO)
2981 Size++;
2982 ElementInfo Info = getAdjustedElementInfo(Layout: BaseLayout);
2983 CharUnits BaseOffset;
2984
2985 // Respect the external AST source base offset, if present.
2986 bool FoundBase = false;
2987 if (UseExternalLayout) {
2988 FoundBase = External.getExternalNVBaseOffset(RD: BaseDecl, BaseOffset);
2989 if (BaseOffset > Size) {
2990 Size = BaseOffset;
2991 }
2992 }
2993
2994 if (!FoundBase) {
2995 if (MDCUsesEBO && BaseDecl->isEmpty() &&
2996 (BaseLayout.getNonVirtualSize() == CharUnits::Zero())) {
2997 BaseOffset = CharUnits::Zero();
2998 } else {
2999 // Otherwise, lay the base out at the end of the MDC.
3000 BaseOffset = Size = Size.alignTo(Align: Info.Alignment);
3001 }
3002 }
3003 Bases.insert(KV: std::make_pair(x&: BaseDecl, y&: BaseOffset));
3004 Size += BaseLayout.getNonVirtualSize();
3005 DataSize = Size;
3006 PreviousBaseLayout = &BaseLayout;
3007}
3008
3009void MicrosoftRecordLayoutBuilder::layoutFields(const RecordDecl *RD) {
3010 LastFieldIsNonZeroWidthBitfield = false;
3011 for (const FieldDecl *Field : RD->fields())
3012 layoutField(FD: Field);
3013}
3014
3015void MicrosoftRecordLayoutBuilder::layoutField(const FieldDecl *FD) {
3016 if (FD->isBitField()) {
3017 layoutBitField(FD);
3018 return;
3019 }
3020 LastFieldIsNonZeroWidthBitfield = false;
3021 ElementInfo Info = getAdjustedElementInfo(FD);
3022 Alignment = std::max(a: Alignment, b: Info.NonRequiredAlignment);
3023
3024 const CXXRecordDecl *FieldClass = FD->getType()->getAsCXXRecordDecl();
3025 bool IsOverlappingEmptyField = FD->isPotentiallyOverlapping() &&
3026 FieldClass->isEmpty() &&
3027 FieldClass->fields().empty();
3028 CharUnits FieldOffset = CharUnits::Zero();
3029
3030 if (UseExternalLayout) {
3031 FieldOffset =
3032 Context.toCharUnitsFromBits(BitSize: External.getExternalFieldOffset(FD));
3033 } else if (IsUnion) {
3034 FieldOffset = CharUnits::Zero();
3035 } else if (EmptySubobjects) {
3036 if (!IsOverlappingEmptyField)
3037 FieldOffset = DataSize.alignTo(Align: Info.Alignment);
3038
3039 while (!EmptySubobjects->CanPlaceFieldAtOffset(FD, Offset: FieldOffset)) {
3040 const CXXRecordDecl *ParentClass = cast<CXXRecordDecl>(Val: FD->getParent());
3041 bool HasBases = ParentClass && (!ParentClass->bases().empty() ||
3042 !ParentClass->vbases().empty());
3043 if (FieldOffset == CharUnits::Zero() && DataSize != CharUnits::Zero() &&
3044 HasBases) {
3045 // MSVC appears to only do this when there are base classes;
3046 // otherwise it overlaps no_unique_address fields in non-zero offsets.
3047 FieldOffset = DataSize.alignTo(Align: Info.Alignment);
3048 } else {
3049 FieldOffset += Info.Alignment;
3050 }
3051 }
3052 } else {
3053 FieldOffset = Size.alignTo(Align: Info.Alignment);
3054 }
3055
3056 uint64_t UnpaddedFielddOffsetInBits =
3057 Context.toBits(CharSize: DataSize) - RemainingBitsInField;
3058
3059 ::CheckFieldPadding(Context, IsUnion, Offset: Context.toBits(CharSize: FieldOffset),
3060 UnpaddedOffset: UnpaddedFielddOffsetInBits, D: FD);
3061
3062 RemainingBitsInField = 0;
3063
3064 placeFieldAtOffset(FieldOffset);
3065
3066 if (!IsOverlappingEmptyField)
3067 DataSize = std::max(a: DataSize, b: FieldOffset + Info.Size);
3068
3069 Size = std::max(a: Size, b: FieldOffset + Info.Size);
3070}
3071
3072void MicrosoftRecordLayoutBuilder::layoutBitField(const FieldDecl *FD) {
3073 unsigned Width = FD->getBitWidthValue();
3074 if (Width == 0) {
3075 layoutZeroWidthBitField(FD);
3076 return;
3077 }
3078 ElementInfo Info = getAdjustedElementInfo(FD);
3079 // Clamp the bitfield to a containable size for the sake of being able
3080 // to lay them out. Sema will throw an error.
3081 if (Width > Context.toBits(CharSize: Info.Size))
3082 Width = Context.toBits(CharSize: Info.Size);
3083 // Check to see if this bitfield fits into an existing allocation. Note:
3084 // MSVC refuses to pack bitfields of formal types with different sizes
3085 // into the same allocation.
3086 if (!UseExternalLayout && !IsUnion && LastFieldIsNonZeroWidthBitfield &&
3087 CurrentBitfieldSize == Info.Size && Width <= RemainingBitsInField) {
3088 placeFieldAtBitOffset(FieldOffset: Context.toBits(CharSize: Size) - RemainingBitsInField);
3089 RemainingBitsInField -= Width;
3090 return;
3091 }
3092 LastFieldIsNonZeroWidthBitfield = true;
3093 CurrentBitfieldSize = Info.Size;
3094 if (UseExternalLayout) {
3095 auto FieldBitOffset = External.getExternalFieldOffset(FD);
3096 placeFieldAtBitOffset(FieldOffset: FieldBitOffset);
3097 auto NewSize = Context.toCharUnitsFromBits(
3098 BitSize: llvm::alignDown(Value: FieldBitOffset, Align: Context.toBits(CharSize: Info.Alignment)) +
3099 Context.toBits(CharSize: Info.Size));
3100 Size = std::max(a: Size, b: NewSize);
3101 Alignment = std::max(a: Alignment, b: Info.NonRequiredAlignment);
3102 } else if (IsUnion) {
3103 placeFieldAtOffset(FieldOffset: CharUnits::Zero());
3104 Size = std::max(a: Size, b: Info.Size);
3105 // TODO: Add a Sema warning that MS ignores bitfield alignment in unions.
3106 } else {
3107 // Allocate a new block of memory and place the bitfield in it.
3108 CharUnits FieldOffset = Size.alignTo(Align: Info.Alignment);
3109 uint64_t UnpaddedFieldOffsetInBits =
3110 Context.toBits(CharSize: DataSize) - RemainingBitsInField;
3111 placeFieldAtOffset(FieldOffset);
3112 Size = FieldOffset + Info.Size;
3113 Alignment = std::max(a: Alignment, b: Info.NonRequiredAlignment);
3114 RemainingBitsInField = Context.toBits(CharSize: Info.Size) - Width;
3115 ::CheckFieldPadding(Context, IsUnion, Offset: Context.toBits(CharSize: FieldOffset),
3116 UnpaddedOffset: UnpaddedFieldOffsetInBits, D: FD);
3117 }
3118 DataSize = Size;
3119}
3120
3121void
3122MicrosoftRecordLayoutBuilder::layoutZeroWidthBitField(const FieldDecl *FD) {
3123 // Zero-width bitfields are ignored unless they follow a non-zero-width
3124 // bitfield.
3125 if (!LastFieldIsNonZeroWidthBitfield) {
3126 placeFieldAtOffset(FieldOffset: IsUnion ? CharUnits::Zero() : Size);
3127 // TODO: Add a Sema warning that MS ignores alignment for zero
3128 // sized bitfields that occur after zero-size bitfields or non-bitfields.
3129 return;
3130 }
3131 LastFieldIsNonZeroWidthBitfield = false;
3132 ElementInfo Info = getAdjustedElementInfo(FD);
3133 if (IsUnion) {
3134 placeFieldAtOffset(FieldOffset: CharUnits::Zero());
3135 Size = std::max(a: Size, b: Info.Size);
3136 // TODO: Add a Sema warning that MS ignores bitfield alignment in unions.
3137 } else {
3138 // Round up the current record size to the field's alignment boundary.
3139 CharUnits FieldOffset = Size.alignTo(Align: Info.Alignment);
3140 uint64_t UnpaddedFieldOffsetInBits =
3141 Context.toBits(CharSize: DataSize) - RemainingBitsInField;
3142 placeFieldAtOffset(FieldOffset);
3143 RemainingBitsInField = 0;
3144 Size = FieldOffset;
3145 Alignment = std::max(a: Alignment, b: Info.NonRequiredAlignment);
3146 ::CheckFieldPadding(Context, IsUnion, Offset: Context.toBits(CharSize: FieldOffset),
3147 UnpaddedOffset: UnpaddedFieldOffsetInBits, D: FD);
3148 }
3149 DataSize = Size;
3150}
3151
3152void MicrosoftRecordLayoutBuilder::injectVBPtr(const CXXRecordDecl *RD) {
3153 if (!HasVBPtr || SharedVBPtrBase)
3154 return;
3155 // Inject the VBPointer at the injection site.
3156 CharUnits InjectionSite = VBPtrOffset;
3157 // But before we do, make sure it's properly aligned.
3158 VBPtrOffset = VBPtrOffset.alignTo(Align: PointerInfo.Alignment);
3159 // Determine where the first field should be laid out after the vbptr.
3160 CharUnits FieldStart = VBPtrOffset + PointerInfo.Size;
3161 // Shift everything after the vbptr down, unless we're using an external
3162 // layout.
3163 if (UseExternalLayout) {
3164 // It is possible that there were no fields or bases located after vbptr,
3165 // so the size was not adjusted before.
3166 if (Size < FieldStart)
3167 Size = FieldStart;
3168 return;
3169 }
3170 // Make sure that the amount we push the fields back by is a multiple of the
3171 // alignment.
3172 CharUnits Offset = (FieldStart - InjectionSite)
3173 .alignTo(Align: std::max(a: RequiredAlignment, b: Alignment));
3174 Size += Offset;
3175 for (uint64_t &FieldOffset : FieldOffsets)
3176 FieldOffset += Context.toBits(CharSize: Offset);
3177 for (BaseOffsetsMapTy::value_type &Base : Bases)
3178 if (Base.second >= InjectionSite)
3179 Base.second += Offset;
3180}
3181
3182void MicrosoftRecordLayoutBuilder::injectVFPtr(const CXXRecordDecl *RD) {
3183 if (!HasOwnVFPtr)
3184 return;
3185 // Make sure that the amount we push the struct back by is a multiple of the
3186 // alignment.
3187 CharUnits Offset =
3188 PointerInfo.Size.alignTo(Align: std::max(a: RequiredAlignment, b: Alignment));
3189 // Push back the vbptr, but increase the size of the object and push back
3190 // regular fields by the offset only if not using external record layout.
3191 if (HasVBPtr)
3192 VBPtrOffset += Offset;
3193
3194 if (UseExternalLayout) {
3195 // The class may have size 0 and a vfptr (e.g. it's an interface class). The
3196 // size was not correctly set before in this case.
3197 if (Size.isZero())
3198 Size += Offset;
3199 return;
3200 }
3201
3202 Size += Offset;
3203
3204 // If we're using an external layout, the fields offsets have already
3205 // accounted for this adjustment.
3206 for (uint64_t &FieldOffset : FieldOffsets)
3207 FieldOffset += Context.toBits(CharSize: Offset);
3208 for (BaseOffsetsMapTy::value_type &Base : Bases)
3209 Base.second += Offset;
3210}
3211
3212void MicrosoftRecordLayoutBuilder::layoutVirtualBases(const CXXRecordDecl *RD) {
3213 if (!HasVBPtr)
3214 return;
3215 // Vtordisps are always 4 bytes (even in 64-bit mode)
3216 CharUnits VtorDispSize = CharUnits::fromQuantity(Quantity: 4);
3217 CharUnits VtorDispAlignment = VtorDispSize;
3218 // vtordisps respect pragma pack.
3219 if (!MaxFieldAlignment.isZero())
3220 VtorDispAlignment = std::min(a: VtorDispAlignment, b: MaxFieldAlignment);
3221 // The alignment of the vtordisp is at least the required alignment of the
3222 // entire record. This requirement may be present to support vtordisp
3223 // injection.
3224 for (const CXXBaseSpecifier &VBase : RD->vbases()) {
3225 const CXXRecordDecl *BaseDecl = VBase.getType()->getAsCXXRecordDecl();
3226 const ASTRecordLayout &BaseLayout = Context.getASTRecordLayout(D: BaseDecl);
3227 RequiredAlignment =
3228 std::max(a: RequiredAlignment, b: BaseLayout.getRequiredAlignment());
3229 }
3230 VtorDispAlignment = std::max(a: VtorDispAlignment, b: RequiredAlignment);
3231 // Compute the vtordisp set.
3232 llvm::SmallPtrSet<const CXXRecordDecl *, 2> HasVtorDispSet;
3233 computeVtorDispSet(HasVtorDispSet, RD);
3234 // Iterate through the virtual bases and lay them out.
3235 const ASTRecordLayout *PreviousBaseLayout = nullptr;
3236 for (const CXXBaseSpecifier &VBase : RD->vbases()) {
3237 const CXXRecordDecl *BaseDecl = VBase.getType()->getAsCXXRecordDecl();
3238 const ASTRecordLayout &BaseLayout = Context.getASTRecordLayout(D: BaseDecl);
3239 bool HasVtordisp = HasVtorDispSet.contains(Ptr: BaseDecl);
3240 // Insert padding between two bases if the left first one is zero sized or
3241 // contains a zero sized subobject and the right is zero sized or one leads
3242 // with a zero sized base. The padding between virtual bases is 4
3243 // bytes (in both 32 and 64 bits modes) and always involves rounding up to
3244 // the required alignment, we don't know why.
3245 if ((PreviousBaseLayout && PreviousBaseLayout->endsWithZeroSizedObject() &&
3246 BaseLayout.leadsWithZeroSizedBase() && !recordUsesEBO(RD)) ||
3247 HasVtordisp) {
3248 Size = Size.alignTo(Align: VtorDispAlignment) + VtorDispSize;
3249 Alignment = std::max(a: VtorDispAlignment, b: Alignment);
3250 }
3251 // Insert the virtual base.
3252 ElementInfo Info = getAdjustedElementInfo(Layout: BaseLayout);
3253 CharUnits BaseOffset;
3254
3255 // Respect the external AST source base offset, if present.
3256 if (UseExternalLayout) {
3257 if (!External.getExternalVBaseOffset(RD: BaseDecl, BaseOffset))
3258 BaseOffset = Size;
3259 } else
3260 BaseOffset = Size.alignTo(Align: Info.Alignment);
3261
3262 assert(BaseOffset >= Size && "base offset already allocated");
3263
3264 VBases.insert(KV: std::make_pair(x&: BaseDecl,
3265 y: ASTRecordLayout::VBaseInfo(BaseOffset, HasVtordisp)));
3266 Size = BaseOffset + BaseLayout.getNonVirtualSize();
3267 PreviousBaseLayout = &BaseLayout;
3268 }
3269}
3270
3271void MicrosoftRecordLayoutBuilder::finalizeLayout(const RecordDecl *RD) {
3272 uint64_t UnpaddedSizeInBits = Context.toBits(CharSize: DataSize);
3273 UnpaddedSizeInBits -= RemainingBitsInField;
3274
3275 // MS ABI allocates 1 byte for empty class
3276 // (not padding)
3277 if (Size.isZero())
3278 UnpaddedSizeInBits += 8;
3279
3280 // Respect required alignment. Note that in 32-bit mode Required alignment
3281 // may be 0 and cause size not to be updated.
3282 DataSize = Size;
3283 // Capture the alignment before folding in the required alignment; this is the
3284 // record's alignment ignoring any over-alignment imposed via alignas /
3285 // __declspec(align).
3286 NonRequiredAlignment = Alignment;
3287 if (!RequiredAlignment.isZero()) {
3288 Alignment = std::max(a: Alignment, b: RequiredAlignment);
3289 auto RoundingAlignment = Alignment;
3290 if (!MaxFieldAlignment.isZero())
3291 RoundingAlignment = std::min(a: RoundingAlignment, b: MaxFieldAlignment);
3292 RoundingAlignment = std::max(a: RoundingAlignment, b: RequiredAlignment);
3293 Size = Size.alignTo(Align: RoundingAlignment);
3294 }
3295 if (Size.isZero()) {
3296 if (!recordUsesEBO(RD) || !cast<CXXRecordDecl>(Val: RD)->isEmpty()) {
3297 EndsWithZeroSizedObject = true;
3298 LeadsWithZeroSizedBase = true;
3299 }
3300 // Zero-sized structures have size equal to their alignment if a
3301 // __declspec(align) came into play.
3302 if (RequiredAlignment >= MinEmptyStructSize)
3303 Size = Alignment;
3304 else
3305 Size = MinEmptyStructSize;
3306 }
3307
3308 if (UseExternalLayout) {
3309 Size = Context.toCharUnitsFromBits(BitSize: External.Size);
3310 if (External.Align) {
3311 Alignment = Context.toCharUnitsFromBits(BitSize: External.Align);
3312 // An external layout provides the record's final alignment, with no way
3313 // to tell how much of it was imposed via alignas / __declspec(align), so
3314 // treat all of it as non-required alignment.
3315 NonRequiredAlignment = Alignment;
3316 }
3317 return;
3318 }
3319 unsigned CharBitNum = Context.getTargetInfo().getCharWidth();
3320 uint64_t SizeInBits = Context.toBits(CharSize: Size);
3321
3322 if (SizeInBits > UnpaddedSizeInBits) {
3323 unsigned int PadSize = SizeInBits - UnpaddedSizeInBits;
3324 bool InBits = true;
3325 if (PadSize % CharBitNum == 0) {
3326 PadSize = PadSize / CharBitNum;
3327 InBits = false;
3328 }
3329
3330 Context.getDiagnostics().Report(Loc: RD->getLocation(),
3331 DiagID: diag::warn_padded_struct_size)
3332 << Context.getCanonicalTagType(TD: RD) << PadSize
3333 << (InBits ? 1 : 0); // (byte|bit)
3334 }
3335}
3336
3337// Recursively walks the non-virtual bases of a class and determines if any of
3338// them are in the bases with overridden methods set.
3339static bool
3340RequiresVtordisp(const llvm::SmallPtrSetImpl<const CXXRecordDecl *> &
3341 BasesWithOverriddenMethods,
3342 const CXXRecordDecl *RD) {
3343 if (BasesWithOverriddenMethods.count(Ptr: RD))
3344 return true;
3345 // If any of a virtual bases non-virtual bases (recursively) requires a
3346 // vtordisp than so does this virtual base.
3347 for (const CXXBaseSpecifier &Base : RD->bases())
3348 if (!Base.isVirtual() &&
3349 RequiresVtordisp(BasesWithOverriddenMethods,
3350 RD: Base.getType()->getAsCXXRecordDecl()))
3351 return true;
3352 return false;
3353}
3354
3355void MicrosoftRecordLayoutBuilder::computeVtorDispSet(
3356 llvm::SmallPtrSetImpl<const CXXRecordDecl *> &HasVtordispSet,
3357 const CXXRecordDecl *RD) const {
3358 // /vd2 or #pragma vtordisp(2): Always use vtordisps for virtual bases with
3359 // vftables.
3360 if (RD->getMSVtorDispMode() == MSVtorDispMode::ForVFTable) {
3361 for (const CXXBaseSpecifier &Base : RD->vbases()) {
3362 const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
3363 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: BaseDecl);
3364 if (Layout.hasExtendableVFPtr())
3365 HasVtordispSet.insert(Ptr: BaseDecl);
3366 }
3367 return;
3368 }
3369
3370 // If any of our bases need a vtordisp for this type, so do we. Check our
3371 // direct bases for vtordisp requirements.
3372 for (const CXXBaseSpecifier &Base : RD->bases()) {
3373 const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
3374 const ASTRecordLayout &Layout = Context.getASTRecordLayout(D: BaseDecl);
3375 for (const auto &bi : Layout.getVBaseOffsetsMap())
3376 if (bi.second.hasVtorDisp())
3377 HasVtordispSet.insert(Ptr: bi.first);
3378 }
3379 // We don't introduce any additional vtordisps if either:
3380 // * A user declared constructor or destructor aren't declared.
3381 // * #pragma vtordisp(0) or the /vd0 flag are in use.
3382 if ((!RD->hasUserDeclaredConstructor() && !RD->hasUserDeclaredDestructor()) ||
3383 RD->getMSVtorDispMode() == MSVtorDispMode::Never)
3384 return;
3385 // /vd1 or #pragma vtordisp(1): Try to guess based on whether we think it's
3386 // possible for a partially constructed object with virtual base overrides to
3387 // escape a non-trivial constructor.
3388 assert(RD->getMSVtorDispMode() == MSVtorDispMode::ForVBaseOverride);
3389 // Compute a set of base classes which define methods we override. A virtual
3390 // base in this set will require a vtordisp. A virtual base that transitively
3391 // contains one of these bases as a non-virtual base will also require a
3392 // vtordisp.
3393 llvm::SmallPtrSet<const CXXMethodDecl *, 8> Work;
3394 llvm::SmallPtrSet<const CXXRecordDecl *, 2> BasesWithOverriddenMethods;
3395 // Seed the working set with our non-destructor, non-pure virtual methods.
3396 for (const CXXMethodDecl *MD : RD->methods())
3397 if (MicrosoftVTableContext::hasVtableSlot(MD) &&
3398 !isa<CXXDestructorDecl>(Val: MD) && !MD->isPureVirtual())
3399 Work.insert(Ptr: MD);
3400 while (!Work.empty()) {
3401 const CXXMethodDecl *MD = *Work.begin();
3402 auto MethodRange = MD->overridden_methods();
3403 // If a virtual method has no-overrides it lives in its parent's vtable.
3404 if (MethodRange.begin() == MethodRange.end())
3405 BasesWithOverriddenMethods.insert(Ptr: MD->getParent());
3406 else
3407 Work.insert_range(R&: MethodRange);
3408 // We've finished processing this element, remove it from the working set.
3409 Work.erase(Ptr: MD);
3410 }
3411 // For each of our virtual bases, check if it is in the set of overridden
3412 // bases or if it transitively contains a non-virtual base that is.
3413 for (const CXXBaseSpecifier &Base : RD->vbases()) {
3414 const CXXRecordDecl *BaseDecl = Base.getType()->getAsCXXRecordDecl();
3415 if (!HasVtordispSet.count(Ptr: BaseDecl) &&
3416 RequiresVtordisp(BasesWithOverriddenMethods, RD: BaseDecl))
3417 HasVtordispSet.insert(Ptr: BaseDecl);
3418 }
3419}
3420
3421bool ASTContext::defaultsToMsStruct() const {
3422 return getTargetInfo().hasMicrosoftRecordLayout() ||
3423 getTargetInfo().getTriple().isWindowsGNUEnvironment();
3424}
3425
3426const ASTRecordLayout &
3427ASTContext::getASTRecordLayout(const RecordDecl *D) const {
3428 if (D->hasExternalLexicalStorage() && !D->getDefinition())
3429 getExternalSource()->CompleteType(Tag: const_cast<RecordDecl*>(D));
3430 // Complete the redecl chain (if necessary).
3431 (void)D->getMostRecentDecl();
3432
3433 // These asserts test different things. A record has a definition
3434 // as soon as we begin to parse the definition. That definition is
3435 // not a complete definition (which is what isCompleteDefinition() tests)
3436 // until we *finish* parsing the definition.
3437 D = D->getDefinition();
3438 assert(D && "Cannot get layout of forward declarations!");
3439 assert(!D->isInvalidDecl() && "Cannot get layout of invalid decl!");
3440 assert(D->isCompleteDefinition() && "Cannot layout type before complete!");
3441
3442 // Look up this layout, if already laid out, return what we have.
3443 // Note that we can't save a reference to the entry because this function
3444 // is recursive.
3445 const ASTRecordLayout *Entry = ASTRecordLayouts[D];
3446 if (Entry) return *Entry;
3447
3448 const ASTRecordLayout *NewEntry = nullptr;
3449
3450 if (getTargetInfo().hasMicrosoftRecordLayout()) {
3451 if (const auto *RD = dyn_cast<CXXRecordDecl>(Val: D)) {
3452 EmptySubobjectMap EmptySubobjects(*this, RD);
3453 MicrosoftRecordLayoutBuilder Builder(*this, &EmptySubobjects);
3454 Builder.cxxLayout(RD);
3455 NewEntry = new (*this) ASTRecordLayout(
3456 *this, Builder.Size, Builder.Alignment, Builder.Alignment,
3457 Builder.Alignment, Builder.RequiredAlignment, Builder.HasOwnVFPtr,
3458 Builder.HasOwnVFPtr || Builder.PrimaryBase, Builder.VBPtrOffset,
3459 Builder.DataSize, Builder.FieldOffsets, Builder.NonVirtualSize,
3460 Builder.Alignment, Builder.Alignment, Builder.NonRequiredAlignment,
3461 CharUnits::Zero(), Builder.PrimaryBase, false,
3462 Builder.SharedVBPtrBase, Builder.EndsWithZeroSizedObject,
3463 Builder.LeadsWithZeroSizedBase, Builder.Bases, Builder.VBases);
3464 } else {
3465 MicrosoftRecordLayoutBuilder Builder(*this, /*EmptySubobjects=*/nullptr);
3466 Builder.layout(RD: D);
3467 NewEntry = new (*this) ASTRecordLayout(
3468 *this, Builder.Size, Builder.Alignment, Builder.Alignment,
3469 Builder.Alignment, Builder.RequiredAlignment, Builder.Size,
3470 Builder.FieldOffsets);
3471 }
3472 } else {
3473 if (const auto *RD = dyn_cast<CXXRecordDecl>(Val: D)) {
3474 EmptySubobjectMap EmptySubobjects(*this, RD);
3475 ItaniumRecordLayoutBuilder Builder(*this, &EmptySubobjects);
3476 Builder.Layout(RD);
3477
3478 // In certain situations, we are allowed to lay out objects in the
3479 // tail-padding of base classes. This is ABI-dependent.
3480 // FIXME: this should be stored in the record layout.
3481 bool skipTailPadding =
3482 mustSkipTailPadding(ABI: getTargetInfo().getCXXABI(), RD);
3483
3484 // FIXME: This should be done in FinalizeLayout.
3485 CharUnits DataSize =
3486 skipTailPadding ? Builder.getSize() : Builder.getDataSize();
3487 CharUnits NonVirtualSize =
3488 skipTailPadding ? DataSize : Builder.NonVirtualSize;
3489 NewEntry = new (*this) ASTRecordLayout(
3490 *this, Builder.getSize(), Builder.Alignment,
3491 Builder.PreferredAlignment, Builder.UnadjustedAlignment,
3492 /*RequiredAlignment : used by MS-ABI)*/
3493 Builder.Alignment, Builder.HasOwnVFPtr, RD->isDynamicClass(),
3494 CharUnits::fromQuantity(Quantity: -1), DataSize, Builder.FieldOffsets,
3495 NonVirtualSize, Builder.NonVirtualAlignment,
3496 Builder.PreferredNVAlignment, Builder.NonVirtualAlignment,
3497 EmptySubobjects.SizeOfLargestEmptySubobject, Builder.PrimaryBase,
3498 Builder.PrimaryBaseIsVirtual, nullptr, false, false, Builder.Bases,
3499 Builder.VBases);
3500 } else {
3501 ItaniumRecordLayoutBuilder Builder(*this, /*EmptySubobjects=*/nullptr);
3502 Builder.Layout(D);
3503
3504 NewEntry = new (*this) ASTRecordLayout(
3505 *this, Builder.getSize(), Builder.Alignment,
3506 Builder.PreferredAlignment, Builder.UnadjustedAlignment,
3507 /*RequiredAlignment : used by MS-ABI)*/
3508 Builder.Alignment, Builder.getSize(), Builder.FieldOffsets);
3509 }
3510 }
3511
3512 ASTRecordLayouts[D] = NewEntry;
3513
3514 // Cap at the target's size_t width (up to 60 bits) so oversized layouts on
3515 // narrow targets are diagnosed instead of overflowing size_t in codegen.
3516 uint64_t MaxStructSizeInBytes =
3517 1ULL << std::min<unsigned>(a: getTypeSize(T: getSizeType()), b: 60);
3518 CharUnits StructSize = NewEntry->getSize();
3519 if (static_cast<uint64_t>(StructSize.getQuantity()) >= MaxStructSizeInBytes) {
3520 getDiagnostics().Report(Loc: D->getLocation(), DiagID: diag::err_struct_too_large)
3521 << D->getName() << MaxStructSizeInBytes;
3522 }
3523
3524 if (getLangOpts().DumpRecordLayouts) {
3525 llvm::outs() << "\n*** Dumping AST Record Layout\n";
3526 DumpRecordLayout(RD: D, OS&: llvm::outs(), Simple: getLangOpts().DumpRecordLayoutsSimple);
3527 }
3528
3529 return *NewEntry;
3530}
3531
3532const CXXMethodDecl *ASTContext::getCurrentKeyFunction(const CXXRecordDecl *RD) {
3533 if (!getTargetInfo().getCXXABI().hasKeyFunctions())
3534 return nullptr;
3535
3536 assert(RD->getDefinition() && "Cannot get key function for forward decl!");
3537 RD = RD->getDefinition();
3538
3539 // Beware:
3540 // 1) computing the key function might trigger deserialization, which might
3541 // invalidate iterators into KeyFunctions
3542 // 2) 'get' on the LazyDeclPtr might also trigger deserialization and
3543 // invalidate the LazyDeclPtr within the map itself
3544 LazyDeclPtr Entry = KeyFunctions[RD];
3545 const Decl *Result =
3546 Entry ? Entry.get(Source: getExternalSource()) : computeKeyFunction(Context&: *this, RD);
3547
3548 // Store it back if it changed.
3549 if (Entry.isOffset() || Entry.isValid() != bool(Result))
3550 KeyFunctions[RD] = const_cast<Decl*>(Result);
3551
3552 return cast_or_null<CXXMethodDecl>(Val: Result);
3553}
3554
3555void ASTContext::setNonKeyFunction(const CXXMethodDecl *Method) {
3556 assert(Method == Method->getFirstDecl() &&
3557 "not working with method declaration from class definition");
3558
3559 // Look up the cache entry. Since we're working with the first
3560 // declaration, its parent must be the class definition, which is
3561 // the correct key for the KeyFunctions hash.
3562 const auto &Map = KeyFunctions;
3563 auto I = Map.find(Val: Method->getParent());
3564
3565 // If it's not cached, there's nothing to do.
3566 if (I == Map.end()) return;
3567
3568 // If it is cached, check whether it's the target method, and if so,
3569 // remove it from the cache. Note, the call to 'get' might invalidate
3570 // the iterator and the LazyDeclPtr object within the map.
3571 LazyDeclPtr Ptr = I->second;
3572 if (Ptr.get(Source: getExternalSource()) == Method) {
3573 // FIXME: remember that we did this for module / chained PCH state?
3574 KeyFunctions.erase(Val: Method->getParent());
3575 }
3576}
3577
3578static uint64_t getFieldOffset(const ASTContext &C, const FieldDecl *FD) {
3579 const ASTRecordLayout &Layout = C.getASTRecordLayout(D: FD->getParent());
3580 return Layout.getFieldOffset(FieldNo: FD->getFieldIndex());
3581}
3582
3583uint64_t ASTContext::getFieldOffset(const ValueDecl *VD) const {
3584 uint64_t OffsetInBits;
3585 if (const FieldDecl *FD = dyn_cast<FieldDecl>(Val: VD)) {
3586 OffsetInBits = ::getFieldOffset(C: *this, FD);
3587 } else {
3588 const IndirectFieldDecl *IFD = cast<IndirectFieldDecl>(Val: VD);
3589
3590 OffsetInBits = 0;
3591 for (const NamedDecl *ND : IFD->chain())
3592 OffsetInBits += ::getFieldOffset(C: *this, FD: cast<FieldDecl>(Val: ND));
3593 }
3594
3595 return OffsetInBits;
3596}
3597
3598uint64_t ASTContext::lookupFieldBitOffset(const ObjCInterfaceDecl *OID,
3599 const ObjCIvarDecl *Ivar) const {
3600 Ivar = Ivar->getCanonicalDecl();
3601 const ObjCInterfaceDecl *Container = Ivar->getContainingInterface();
3602 const ASTRecordLayout *RL = &getASTObjCInterfaceLayout(D: Container);
3603
3604 // Compute field index.
3605 //
3606 // FIXME: The index here is closely tied to how ASTContext::getObjCLayout is
3607 // implemented. This should be fixed to get the information from the layout
3608 // directly.
3609 unsigned Index = 0;
3610
3611 for (const ObjCIvarDecl *IVD = Container->all_declared_ivar_begin();
3612 IVD; IVD = IVD->getNextIvar()) {
3613 if (Ivar == IVD)
3614 break;
3615 ++Index;
3616 }
3617 assert(Index < RL->getFieldCount() && "Ivar is not inside record layout!");
3618
3619 return RL->getFieldOffset(FieldNo: Index);
3620}
3621
3622/// getObjCLayout - Get or compute information about the layout of the
3623/// given interface.
3624///
3625/// \param Impl - If given, also include the layout of the interface's
3626/// implementation. This may differ by including synthesized ivars.
3627const ASTRecordLayout &
3628ASTContext::getObjCLayout(const ObjCInterfaceDecl *D) const {
3629 // Retrieve the definition
3630 if (D->hasExternalLexicalStorage() && !D->getDefinition())
3631 getExternalSource()->CompleteType(Class: const_cast<ObjCInterfaceDecl*>(D));
3632 D = D->getDefinition();
3633 assert(D && !D->isInvalidDecl() && D->isThisDeclarationADefinition() &&
3634 "Invalid interface decl!");
3635
3636 // Look up this layout, if already laid out, return what we have.
3637 if (const ASTRecordLayout *Entry = ObjCLayouts[D])
3638 return *Entry;
3639
3640 ItaniumRecordLayoutBuilder Builder(*this, /*EmptySubobjects=*/nullptr);
3641 Builder.Layout(D);
3642
3643 const ASTRecordLayout *NewEntry = new (*this) ASTRecordLayout(
3644 *this, Builder.getSize(), Builder.Alignment, Builder.PreferredAlignment,
3645 Builder.UnadjustedAlignment,
3646 /*RequiredAlignment : used by MS-ABI)*/
3647 Builder.Alignment, Builder.getDataSize(), Builder.FieldOffsets);
3648
3649 ObjCLayouts[D] = NewEntry;
3650
3651 return *NewEntry;
3652}
3653
3654static void PrintOffset(raw_ostream &OS,
3655 CharUnits Offset, unsigned IndentLevel) {
3656 OS << llvm::format(Fmt: "%10" PRId64 " | ", Vals: (int64_t)Offset.getQuantity());
3657 OS.indent(NumSpaces: IndentLevel * 2);
3658}
3659
3660static void PrintBitFieldOffset(raw_ostream &OS, CharUnits Offset,
3661 unsigned Begin, unsigned Width,
3662 unsigned IndentLevel) {
3663 llvm::SmallString<10> Buffer;
3664 {
3665 llvm::raw_svector_ostream BufferOS(Buffer);
3666 BufferOS << Offset.getQuantity() << ':';
3667 if (Width == 0) {
3668 BufferOS << '-';
3669 } else {
3670 BufferOS << Begin << '-' << (Begin + Width - 1);
3671 }
3672 }
3673
3674 OS << llvm::right_justify(Str: Buffer, Width: 10) << " | ";
3675 OS.indent(NumSpaces: IndentLevel * 2);
3676}
3677
3678static void PrintIndentNoOffset(raw_ostream &OS, unsigned IndentLevel) {
3679 OS << " | ";
3680 OS.indent(NumSpaces: IndentLevel * 2);
3681}
3682
3683static void DumpRecordLayout(raw_ostream &OS, const RecordDecl *RD,
3684 const ASTContext &C,
3685 CharUnits Offset,
3686 unsigned IndentLevel,
3687 const char* Description,
3688 bool PrintSizeInfo,
3689 bool IncludeVirtualBases) {
3690 const ASTRecordLayout &Layout = C.getASTRecordLayout(D: RD);
3691 auto CXXRD = dyn_cast<CXXRecordDecl>(Val: RD);
3692
3693 PrintOffset(OS, Offset, IndentLevel);
3694 OS << C.getCanonicalTagType(TD: const_cast<RecordDecl *>(RD));
3695 if (Description)
3696 OS << ' ' << Description;
3697 if (CXXRD && CXXRD->isEmpty())
3698 OS << " (empty)";
3699 OS << '\n';
3700
3701 IndentLevel++;
3702
3703 // Dump bases.
3704 if (CXXRD) {
3705 const CXXRecordDecl *PrimaryBase = Layout.getPrimaryBase();
3706 bool HasOwnVFPtr = Layout.hasOwnVFPtr();
3707 bool HasOwnVBPtr = Layout.hasOwnVBPtr();
3708
3709 // Vtable pointer.
3710 if (CXXRD->isDynamicClass() && !PrimaryBase &&
3711 !C.getTargetInfo().hasMicrosoftRecordLayout()) {
3712 PrintOffset(OS, Offset, IndentLevel);
3713 OS << '(' << *RD << " vtable pointer)\n";
3714 } else if (HasOwnVFPtr) {
3715 PrintOffset(OS, Offset, IndentLevel);
3716 // vfptr (for Microsoft C++ ABI)
3717 OS << '(' << *RD << " vftable pointer)\n";
3718 }
3719
3720 // Collect nvbases.
3721 SmallVector<const CXXRecordDecl *, 4> Bases;
3722 for (const CXXBaseSpecifier &Base : CXXRD->bases()) {
3723 assert(!Base.getType()->isDependentType() &&
3724 "Cannot layout class with dependent bases.");
3725 if (!Base.isVirtual())
3726 Bases.push_back(Elt: Base.getType()->getAsCXXRecordDecl());
3727 }
3728
3729 // Sort nvbases by offset.
3730 llvm::stable_sort(
3731 Range&: Bases, C: [&](const CXXRecordDecl *L, const CXXRecordDecl *R) {
3732 return Layout.getBaseClassOffset(Base: L) < Layout.getBaseClassOffset(Base: R);
3733 });
3734
3735 // Dump (non-virtual) bases
3736 for (const CXXRecordDecl *Base : Bases) {
3737 CharUnits BaseOffset = Offset + Layout.getBaseClassOffset(Base);
3738 DumpRecordLayout(OS, RD: Base, C, Offset: BaseOffset, IndentLevel,
3739 Description: Base == PrimaryBase ? "(primary base)" : "(base)",
3740 /*PrintSizeInfo=*/false,
3741 /*IncludeVirtualBases=*/false);
3742 }
3743
3744 // vbptr (for Microsoft C++ ABI)
3745 if (HasOwnVBPtr) {
3746 PrintOffset(OS, Offset: Offset + Layout.getVBPtrOffset(), IndentLevel);
3747 OS << '(' << *RD << " vbtable pointer)\n";
3748 }
3749 }
3750
3751 // Dump fields.
3752 for (const FieldDecl *Field : RD->fields()) {
3753 uint64_t LocalFieldOffsetInBits =
3754 Layout.getFieldOffset(FieldNo: Field->getFieldIndex());
3755 CharUnits FieldOffset =
3756 Offset + C.toCharUnitsFromBits(BitSize: LocalFieldOffsetInBits);
3757
3758 // Recursively dump fields of record type.
3759 if (const auto *RD = Field->getType()->getAsRecordDecl()) {
3760 DumpRecordLayout(OS, RD, C, Offset: FieldOffset, IndentLevel,
3761 Description: Field->getName().data(),
3762 /*PrintSizeInfo=*/false,
3763 /*IncludeVirtualBases=*/true);
3764 continue;
3765 }
3766
3767 if (Field->isBitField()) {
3768 uint64_t LocalFieldByteOffsetInBits = C.toBits(CharSize: FieldOffset - Offset);
3769 unsigned Begin = LocalFieldOffsetInBits - LocalFieldByteOffsetInBits;
3770 unsigned Width = Field->getBitWidthValue();
3771 PrintBitFieldOffset(OS, Offset: FieldOffset, Begin, Width, IndentLevel);
3772 } else {
3773 PrintOffset(OS, Offset: FieldOffset, IndentLevel);
3774 }
3775 const QualType &FieldType = C.getLangOpts().DumpRecordLayoutsCanonical
3776 ? Field->getType().getCanonicalType()
3777 : Field->getType();
3778 OS << FieldType << ' ' << *Field << '\n';
3779 }
3780
3781 // Dump virtual bases.
3782 if (CXXRD && IncludeVirtualBases) {
3783 const ASTRecordLayout::VBaseOffsetsMapTy &VtorDisps =
3784 Layout.getVBaseOffsetsMap();
3785
3786 for (const CXXBaseSpecifier &Base : CXXRD->vbases()) {
3787 assert(Base.isVirtual() && "Found non-virtual class!");
3788 const CXXRecordDecl *VBase = Base.getType()->getAsCXXRecordDecl();
3789
3790 CharUnits VBaseOffset = Offset + Layout.getVBaseClassOffset(VBase);
3791
3792 if (VtorDisps.find(Val: VBase)->second.hasVtorDisp()) {
3793 PrintOffset(OS, Offset: VBaseOffset - CharUnits::fromQuantity(Quantity: 4), IndentLevel);
3794 OS << "(vtordisp for vbase " << *VBase << ")\n";
3795 }
3796
3797 DumpRecordLayout(OS, RD: VBase, C, Offset: VBaseOffset, IndentLevel,
3798 Description: VBase == Layout.getPrimaryBase() ?
3799 "(primary virtual base)" : "(virtual base)",
3800 /*PrintSizeInfo=*/false,
3801 /*IncludeVirtualBases=*/false);
3802 }
3803 }
3804
3805 if (!PrintSizeInfo) return;
3806
3807 PrintIndentNoOffset(OS, IndentLevel: IndentLevel - 1);
3808 OS << "[sizeof=" << Layout.getSize().getQuantity();
3809 if (CXXRD && !C.getTargetInfo().hasMicrosoftRecordLayout())
3810 OS << ", dsize=" << Layout.getDataSize().getQuantity();
3811 OS << ", align=" << Layout.getAlignment().getQuantity();
3812 if (C.getTargetInfo().defaultsToAIXPowerAlignment())
3813 OS << ", preferredalign=" << Layout.getPreferredAlignment().getQuantity();
3814
3815 if (CXXRD) {
3816 OS << ",\n";
3817 PrintIndentNoOffset(OS, IndentLevel: IndentLevel - 1);
3818 OS << " nvsize=" << Layout.getNonVirtualSize().getQuantity();
3819 OS << ", nvalign=" << Layout.getNonVirtualAlignment().getQuantity();
3820 if (C.getTargetInfo().defaultsToAIXPowerAlignment())
3821 OS << ", preferrednvalign="
3822 << Layout.getPreferredNVAlignment().getQuantity();
3823 }
3824 OS << "]\n";
3825}
3826
3827void ASTContext::DumpRecordLayout(const RecordDecl *RD, raw_ostream &OS,
3828 bool Simple) const {
3829 if (!Simple) {
3830 ::DumpRecordLayout(OS, RD, C: *this, Offset: CharUnits(), IndentLevel: 0, Description: nullptr,
3831 /*PrintSizeInfo*/ true,
3832 /*IncludeVirtualBases=*/true);
3833 return;
3834 }
3835
3836 // The "simple" format is designed to be parsed by the
3837 // layout-override testing code. There shouldn't be any external
3838 // uses of this format --- when LLDB overrides a layout, it sets up
3839 // the data structures directly --- so feel free to adjust this as
3840 // you like as long as you also update the rudimentary parser for it
3841 // in libFrontend.
3842
3843 const ASTRecordLayout &Info = getASTRecordLayout(D: RD);
3844 OS << "Type: " << getCanonicalTagType(TD: RD) << "\n";
3845 OS << "\nLayout: ";
3846 OS << "<ASTRecordLayout\n";
3847 OS << " Size:" << toBits(CharSize: Info.getSize()) << "\n";
3848 if (!getTargetInfo().hasMicrosoftRecordLayout())
3849 OS << " DataSize:" << toBits(CharSize: Info.getDataSize()) << "\n";
3850 OS << " Alignment:" << toBits(CharSize: Info.getAlignment()) << "\n";
3851 if (Target->defaultsToAIXPowerAlignment())
3852 OS << " PreferredAlignment:" << toBits(CharSize: Info.getPreferredAlignment())
3853 << "\n";
3854 if (const CXXRecordDecl *CXXRD = dyn_cast<CXXRecordDecl>(Val: RD)) {
3855 OS << " BaseOffsets: [";
3856 const CXXRecordDecl *Base = nullptr;
3857 for (auto I : CXXRD->bases()) {
3858 if (I.isVirtual())
3859 continue;
3860 if (Base)
3861 OS << ", ";
3862 Base = I.getType()->getAsCXXRecordDecl();
3863 OS << Info.CXXInfo->BaseOffsets[Base].getQuantity();
3864 }
3865 OS << "]>\n";
3866 OS << " VBaseOffsets: [";
3867 const CXXRecordDecl *VBase = nullptr;
3868 for (auto I : CXXRD->vbases()) {
3869 if (VBase)
3870 OS << ", ";
3871 VBase = I.getType()->getAsCXXRecordDecl();
3872 OS << Info.CXXInfo->VBaseOffsets[VBase].VBaseOffset.getQuantity();
3873 }
3874 OS << "]>\n";
3875 }
3876 OS << " FieldOffsets: [";
3877 for (unsigned i = 0, e = Info.getFieldCount(); i != e; ++i) {
3878 if (i)
3879 OS << ", ";
3880 OS << Info.getFieldOffset(FieldNo: i);
3881 }
3882 OS << "]>\n";
3883}
3884